@@ -1,12 +1,12 @@
-/* @(#)cdr_drv.c	1.28 02/09/25 Copyright 1997 J. Schilling */
+/* @(#)cdr_drv.c	1.36 04/03/02 Copyright 1997-2004 J. Schilling */
 #ifndef lint
 static	char sccsid[] =
-	"@(#)cdr_drv.c	1.28 02/09/25 Copyright 1997 J. Schilling";
+	"@(#)cdr_drv.c	1.36 04/03/02 Copyright 1997-2004 J. Schilling";
 #endif
 /*
  *	CDR device abstraction layer
  *
- *	Copyright (c) 1997 J. Schilling
+ *	Copyright (c) 1997-2004 J. Schilling
  */
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -19,9 +19,9 @@ static	char sccsid[] =
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License along with
+ * this program; see the file COPYING.  If not, write to the Free Software
+ * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #include <mconfig.h>
@@ -57,6 +57,7 @@ extern	cdr_t	cdr_sony_cdu924;
 extern	cdr_t	cdr_ricoh_ro1060;
 extern	cdr_t	cdr_ricoh_ro1420;
 extern	cdr_t	cdr_teac_cdr50;
+extern	cdr_t	cdr_cw7501;
 extern	cdr_t	cdr_cdr_simul;
 extern	cdr_t	cdr_dvd_simul;
 
@@ -64,10 +65,11 @@ EXPORT	cdr_t 	*drive_identify		__PR((SCSI *scgp, cdr_t *, struct scsi_inquiry *i
 EXPORT	int	drive_attach		__PR((SCSI *scgp, cdr_t *));
 EXPORT	int	attach_unknown		__PR((void));
 EXPORT	int	blank_dummy		__PR((SCSI *scgp, cdr_t *, long addr, int blanktype));
+EXPORT	int	format_dummy		__PR((SCSI *scgp, cdr_t *, int fmtflags));
 EXPORT	int	drive_getdisktype	__PR((SCSI *scgp, cdr_t *dp));
 EXPORT	int	cmd_ill			__PR((SCSI *scgp));
-EXPORT	int	cmd_dummy		__PR((SCSI *scgp));
-EXPORT	int	no_sendcue		__PR((SCSI *scgp, track_t *trackp));
+EXPORT	int	cmd_dummy		__PR((SCSI *scgp, cdr_t *));
+EXPORT	int	no_sendcue		__PR((SCSI *scgp, cdr_t *, track_t *trackp));
 EXPORT	int	buf_dummy		__PR((SCSI *scgp, long *sp, long *fp));
 EXPORT	BOOL	set_cdrcmds		__PR((char *name, cdr_t **dpp));
 EXPORT	cdr_t	*get_cdrcmds		__PR((SCSI *scgp));
@@ -94,6 +96,7 @@ cdr_t	*drivers[] = {
 	&cdr_ricoh_ro1420,
 	&cdr_sony_cdu924,
 	&cdr_teac_cdr50,
+	&cdr_cw7501,
 	&cdr_cdr_simul,
 	&cdr_dvd_simul,
 	(cdr_t *)NULL,
@@ -130,7 +133,17 @@ blank_dummy(scgp, dp, addr, blanktype)
 	long	addr;
 	int	blanktype;
 {
-	printf("This drive or media does not support the BLANK command\n");
+	printf("This drive or media does not support the 'BLANK media' command\n");
+	return (-1);
+}
+
+EXPORT int
+format_dummy(scgp, dp, fmtflags)
+	SCSI	*scgp;
+	cdr_t	*dp;
+	int	fmtflags;
+{
+	printf("This drive or media does not support the 'FORMAT media' command\n");
 	return (-1);
 }
 
@@ -152,15 +165,17 @@ cmd_ill(scgp)
 }
 
 EXPORT int
-cmd_dummy(scgp)
+cmd_dummy(scgp, dp)
 	SCSI	*scgp;
+	cdr_t	*dp;
 {
 	return (0);
 }
 
 EXPORT int
-no_sendcue(scgp, trackp)
+no_sendcue(scgp, dp, trackp)
 	SCSI	*scgp;
+	cdr_t	*dp;
 	track_t	*trackp;
 {
 	errmsgno(EX_BAD, "SAO writing not available or not implemented for this drive.\n");
@@ -217,9 +232,43 @@ get_cdrcmds(scgp)
 {
 	cdr_t	*dp = (cdr_t *)0;
 	cdr_t	*odp = (cdr_t *)0;
+	BOOL	is_wr = FALSE;
+	BOOL	is_cd = FALSE;
+	BOOL	is_dvd = FALSE;
+	BOOL	is_dvdplus = FALSE;
+	BOOL	is_ddcd = FALSE;
 	BOOL	is_cdwr = FALSE;
 	BOOL	is_dvdwr = FALSE;
+	BOOL	is_dvdpluswr = FALSE;
+	BOOL	is_ddcdwr = FALSE;
 
+	/*
+	 * First check for SCSI-3/mmc-3 drives.
+	 */
+	if (get_proflist(scgp, &is_wr, &is_cd, &is_dvd,
+						&is_dvdplus, &is_ddcd) >= 0) {
+
+		get_wproflist(scgp, &is_cdwr, &is_dvdwr,
+						&is_dvdpluswr, &is_ddcdwr);
+		if (xdebug) {
+			error(
+			"Found MMC-3 %s CD: %s/%s DVD-: %s/%s DVD+: %s/%s DDCD: %s/%s.\n",
+					is_wr ? "writer": "reader",
+					is_cd?"r":"-",
+					is_cdwr?"w":"-",
+					is_dvd?"r":"-",
+					is_dvdwr?"w":"-",
+					is_dvdplus?"r":"-",
+					is_dvdpluswr?"w":"-",
+					is_ddcd?"r":"-",
+					is_ddcdwr?"w":"-");
+		}
+		if (!is_wr) {
+			dp = &cdr_cd;
+		} else {
+			dp = &cdr_cd_dvd;
+		}
+	} else
 	/*
 	 * First check for SCSI-3/mmc drives.
 	 */
@@ -250,6 +299,7 @@ get_cdrcmds(scgp)
 	case DEV_TYUDEN_EW50:	dp = &cdr_tyuden_ew50;		break;
 	case DEV_PCD_600:	dp = &cdr_kodak_pcd600;		break;
 	case DEV_YAMAHA_CDR_100:dp = &cdr_yamaha_cdr100;	break;
+	case DEV_MATSUSHITA_7501:dp = &cdr_cw7501;		break;
 	case DEV_MATSUSHITA_7502:
 	case DEV_YAMAHA_CDR_400:dp = &cdr_mmc;			break;
 	case DEV_PLASMON_RF_4100:dp = &cdr_plasmon_rf4100;	break;
@@ -265,14 +315,20 @@ get_cdrcmds(scgp)
 	odp = dp;
 
 	if (xdebug) {
-		error("Using driver '%s' for identify.\n", dp->cdr_drname);
+		error("Using driver '%s' for identify.\n",
+			dp != NULL ?
+			dp->cdr_drname :
+			"<no driver>");
 	}
 
 	if (dp != (cdr_t *)0)
 		dp = dp->cdr_identify(scgp, dp, scgp->inq);
 
 	if (xdebug && dp != odp) {
-		error("Identify set driver to '%s'.\n", dp->cdr_drname);
+		error("Identify set driver to '%s'.\n",
+			dp != NULL ?
+			dp->cdr_drname :
+			"<no driver>");
 	}
 
 	return (dp);
