@@ -1,13 +1,13 @@
-/* @(#)drv_sony.c	1.59 02/09/24 Copyright 1997 J. Schilling */
+/* @(#)drv_sony.c	1.70 04/03/02 Copyright 1997-2004 J. Schilling */
 #ifndef lint
 static	char sccsid[] =
-	"@(#)drv_sony.c	1.59 02/09/24 Copyright 1997 J. Schilling";
+	"@(#)drv_sony.c	1.70 04/03/02 Copyright 1997-2004 J. Schilling";
 #endif
 /*
  *	CDR device implementation for
  *	Sony
  *
- *	Copyright (c) 1997 J. Schilling
+ *	Copyright (c) 1997-2004 J. Schilling
  */
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -20,9 +20,9 @@ static	char sccsid[] =
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License along with
+ * this program; see the file COPYING.  If not, write to the Free Software
+ * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 /*#define	SONY_DEBUG*/
@@ -30,11 +30,13 @@ static	char sccsid[] =
 #include <mconfig.h>
 
 #include <stdio.h>
+#include <stdxlib.h>
 #include <unixstd.h>	/* Include sys/types.h to make off_t available */
 #include <standard.h>
 #include <fctldefs.h>
 #include <errno.h>
 #include <strdefs.h>
+#include <timedefs.h>
 
 #include <utypes.h>
 #include <btorder.h>
@@ -56,6 +58,7 @@ static	char sccsid[] =
 #	define		dec_verbose()
 #endif
 
+extern	int	debug;
 extern	int	lverbose;
 
 #if defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */
@@ -151,21 +154,78 @@ struct cdd_52x_mode_data {
 	} pagex;
 };
 
+struct sony_write_parameter {
+	Uchar	res0;			/* Reserved (must be zero)	*/
+	Uchar	len;			/* Parameter length 0x32 == 52	*/
+	Uchar	res2;			/* Reserved (must be zero)	*/
+#if defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */
+	Ucbit	res3_05		: 6;	/* Reserved			*/
+	Ucbit	ms		: 2;	/* Multi session mode		*/
+#else				/* Motorola byteorder */
+	Ucbit	ms		: 2;	/* Multi session mode		*/
+	Ucbit	res3_05		: 6;	/* Reserved			*/
+#endif
+	Uchar	resx[12];
+#if defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */
+	Ucbit	res16_06	: 7;	/* Reserved			*/
+	Ucbit	mcval		: 1;	/* MCN valid			*/
+#else				/* Motorola byteorder */
+	Ucbit	mcval		: 1;	/* MCN valid			*/
+	Ucbit	res16_06	: 7;	/* Reserved			*/
+#endif
+	Uchar	mcn[15];
+#if defined(_BIT_FIELDS_LTOH)	/* Intel byteorder */
+	Ucbit	res32_06	: 7;	/* Reserved			*/
+	Ucbit	icval		: 1;	/* ISRC valid			*/
+#else				/* Motorola byteorder */
+	Ucbit	icval		: 1;	/* ISRC valid			*/
+	Ucbit	res32_06	: 7;	/* Reserved			*/
+#endif
+	Uchar	isrc[15];
+	Uchar	subheader[4];
+};
+
+struct sony_cue {
+	Uchar	cs_ctladr;		/* CTL/ADR for this track	*/
+	Uchar	cs_tno;			/* This track number		*/
+	Uchar	cs_index;		/* Index within this track	*/
+	Uchar	cs_dataform;		/* Data form 			*/
+					/* Bit 0..5 Main channel Format	*/
+					/* Bit 6..7 SubChannel format	*/
+	Uchar	cs_zero;		/* Reserved or MCN/ISRC		*/
+	Uchar	cs_min;			/* Absolute time minutes	*/
+	Uchar	cs_sec;			/* Absolute time seconds	*/
+	Uchar	cs_frame;		/* Absolute time frames		*/
+};
+
+
+#define	strbeg(s1, s2)	(strstr((s2), (s1)) == (s2))
+
+LOCAL	int	write_start_sony	__PR((SCSI *scgp, caddr_t bp, int size));
 LOCAL	int	write_continue_sony	__PR((SCSI *scgp, caddr_t bp, long sectaddr, long size, int blocks, BOOL islast));
+LOCAL	int	discontinue_sony	__PR((SCSI *scgp));
 LOCAL	int	write_track_sony	__PR((SCSI *scgp, long track, int sectype));
 LOCAL	int	close_track_sony	__PR((SCSI *scgp, cdr_t *dp, track_t *trackp));
-LOCAL	int	finalize_sony		__PR((SCSI *scgp, cdr_t *dp, int onp, int dummy, int type, track_t *trackp));
-LOCAL	int	recover_sony		__PR((SCSI *scgp, int track));
+LOCAL	int	flush_sony		__PR((SCSI *scgp, int track));
+LOCAL	int	finalize_sony		__PR((SCSI *scgp, cdr_t *dp, track_t *trackp));
+LOCAL	int	recover_sony		__PR((SCSI *scgp, cdr_t *dp, int track));
+LOCAL	int	set_wr_parameter_sony	__PR((SCSI *scgp, caddr_t bp, int size));
 LOCAL	int	next_wr_addr_sony	__PR((SCSI *scgp, track_t *trackp, long *ap));
 LOCAL	int	reserve_track_sony	__PR((SCSI *scgp, unsigned long len));
+LOCAL	int	init_sony		__PR((SCSI *scgp, cdr_t *dp));
 LOCAL	int	getdisktype_sony	__PR((SCSI *scgp, cdr_t *dp));
 LOCAL	void	di_to_dstat_sony	__PR((struct sony_924_mode_page_22 *dip, dstat_t *dsp));
-LOCAL	int	speed_select_sony	__PR((SCSI *scgp, cdr_t *dp, int *speedp, int dummy));
+LOCAL	int	speed_select_sony	__PR((SCSI *scgp, cdr_t *dp, int *speedp));
 LOCAL	int	next_writable_address_sony __PR((SCSI *scgp, long *ap, int track, int sectype, int tracktype));
 LOCAL	int	new_track_sony		__PR((SCSI *scgp, int track, int sectype, int tracktype));
 LOCAL	int	open_track_sony		__PR((SCSI *scgp, cdr_t *dp, track_t *trackp));
-LOCAL	int	open_session_sony	__PR((SCSI *scgp, cdr_t *dp, track_t *trackp, int toctype, int multi));
+LOCAL	int	open_session_sony	__PR((SCSI *scgp, cdr_t *dp, track_t *trackp));
+LOCAL	int	abort_session_sony	__PR((SCSI *scgp, cdr_t *dp));
 LOCAL	int	get_page22_sony		__PR((SCSI *scgp, char *mode));
+LOCAL	int	gen_cue_sony		__PR((track_t *trackp, void *vcuep, BOOL needgap));
+LOCAL	void	fillcue			__PR((struct sony_cue *cp, int ca, int tno, int idx, int dataform, int scms, msf_t *mp));
+LOCAL	int	send_cue_sony		__PR((SCSI *scgp, cdr_t *dp, track_t *trackp));
+LOCAL	int	write_leadin_sony	__PR((SCSI *scgp, cdr_t *dp, track_t *trackp));
 LOCAL	int	sony_attach		__PR((SCSI *scgp, cdr_t *dp));
 #ifdef	SONY_DEBUG
 LOCAL	void	print_sony_mp22		__PR((struct sony_924_mode_page_22 *xp, int len));
@@ -175,40 +235,70 @@ LOCAL	int	buf_cap_sony		__PR((SCSI *scgp, long *, long *));
 
 cdr_t	cdr_sony_cdu924 = {
 	0, 0,
-/*	CDR_TAO|CDR_SAO|CDR_CADDYLOAD|CDR_SWABAUDIO,*/
-	CDR_TAO|CDR_CADDYLOAD|CDR_SWABAUDIO,
+	CDR_TAO|CDR_SAO|CDR_CADDYLOAD|CDR_SWABAUDIO,
 	2, 4,
 	"sony_cdu924",
-	"driver for Sony CDU-924",
+	"driver for Sony CDU-924 / CDU-948",
 	0,
 	(dstat_t *)0,
 	drive_identify,
 	sony_attach,
+	init_sony,
 	getdisktype_sony,
 	scsi_load,
 	scsi_unload,
 	buf_cap_sony,
-	(int(*)__PR((SCSI *)))cmd_dummy,	/* recovery_needed	*/
+	cmd_dummy,					/* recovery_needed */
 	recover_sony,
 	speed_select_sony,
 	select_secsize,
 	next_wr_addr_sony,
 	reserve_track_sony,
 	write_continue_sony,
-	no_sendcue,
-	(int(*)__PR((SCSI *, cdr_t *, track_t *)))cmd_dummy, /* leadin */
+	gen_cue_sony,
+	send_cue_sony,
+	write_leadin_sony,
 	open_track_sony,
 	close_track_sony,
 	open_session_sony,
 	cmd_dummy,
+	abort_session_sony,
 	read_session_offset_philips,
 	finalize_sony,
-	(int(*)__PR((SCSI *, cdr_t *)))cmd_dummy,/* stats		*/
+	cmd_dummy,					/* stats	*/
 	blank_dummy,
+	format_dummy,
 	(int(*)__PR((SCSI *, caddr_t, int, int)))NULL,	/* no OPC	*/
-	(int(*)__PR((SCSI *, cdr_t *)))cmd_dummy,/* opt1		*/
+	cmd_dummy,					/* opt1		*/
+	cmd_dummy,					/* opt2		*/
 };
 
+LOCAL int
+write_start_sony(scgp, bp, size)
+	SCSI	*scgp;
+	caddr_t	bp;		/* address of CUE buffer */
+	int	size;		/* number of bytes in CUE buffer */
+{
+	register struct	scg_cmd	*scmd = scgp->scmd;
+
+	fillbytes((caddr_t)scmd, sizeof (*scmd), '\0');
+	scmd->addr = bp;
+	scmd->size = size;
+	scmd->flags = SCG_DISRE_ENA|SCG_CMD_RETRY;
+	scmd->cdb_len = SC_G1_CDBLEN;
+	scmd->sense_len = CCS_SENSE_LEN;
+	scmd->sense_len = 26;
+	scmd->cdb.g1_cdb.cmd = 0xE0;
+	scmd->cdb.g1_cdb.lun = scg_lun(scgp);
+	g0_cdbaddr(&scmd->cdb.g0_cdb, size); /* Hack, but Sony is silly */
+
+	scgp->cmdname = "write_start";
+
+	if (scg_cmd(scgp) < 0)
+		return (-1);
+	return (0);
+}
+
 LOCAL int
 write_continue_sony(scgp, bp, sectaddr, size, blocks, islast)
 	SCSI	*scgp;
@@ -220,7 +310,7 @@ write_continue_sony(scgp, bp, sectaddr, size, blocks, islast)
 {
 	register struct	scg_cmd	*scmd = scgp->scmd;
 
-	fillbytes((caddr_t)scmd, sizeof(*scmd), '\0');
+	fillbytes((caddr_t)scmd, sizeof (*scmd), '\0');
 	scmd->addr = bp;
 	scmd->size = size;
 	scmd->flags = SCG_DISRE_ENA|SCG_CMD_RETRY;
@@ -229,12 +319,37 @@ write_continue_sony(scgp, bp, sectaddr, size, blocks, islast)
 	scmd->cdb.g1_cdb.cmd = 0xE1;
 	scmd->cdb.g1_cdb.lun = scg_lun(scgp);
 	g0_cdbaddr(&scmd->cdb.g0_cdb, size); /* Hack, but Sony is silly */
-	
+
 	scgp->cmdname = "write_continue";
 
+	if (scg_cmd(scgp) < 0) {
+		/*
+		 * XXX This seems to happen only sometimes.
+		 */
+		if (scg_sense_code(scgp) != 0x80)
+			return (-1);
+	}
+	return (size - scg_getresid(scgp));
+}
+
+LOCAL int
+discontinue_sony(scgp)
+	SCSI	*scgp;
+{
+	register struct	scg_cmd	*scmd = scgp->scmd;
+
+	fillbytes((caddr_t)scmd, sizeof (*scmd), '\0');
+	scmd->flags = SCG_DISRE_ENA|SCG_CMD_RETRY;
+	scmd->cdb_len = SC_G1_CDBLEN;
+	scmd->sense_len = CCS_SENSE_LEN;
+	scmd->cdb.g1_cdb.cmd = 0xE2;
+	scmd->cdb.g1_cdb.lun = scg_lun(scgp);
+
+	scgp->cmdname = "discontinue";
+
 	if (scg_cmd(scgp) < 0)
 		return (-1);
-	return (size - scg_getresid(scgp));
+	return (0);
 }
 
 LOCAL int
@@ -245,14 +360,14 @@ write_track_sony(scgp, track, sectype)
 {
 	register struct	scg_cmd	*scmd = scgp->scmd;
 
-	fillbytes((caddr_t)scmd, sizeof(*scmd), '\0');
+	fillbytes((caddr_t)scmd, sizeof (*scmd), '\0');
 	scmd->flags = SCG_DISRE_ENA|SCG_CMD_RETRY;
 	scmd->cdb_len = SC_G1_CDBLEN;
 	scmd->sense_len = CCS_SENSE_LEN;
 	scmd->cdb.g1_cdb.cmd = 0xF5;
 	scmd->cdb.g1_cdb.lun = scg_lun(scgp);
 	g1_cdbaddr(&scmd->cdb.g1_cdb, track);
-	
+
 	scgp->cmdname = "write_track";
 
 	if (scg_cmd(scgp) < 0)
@@ -270,15 +385,18 @@ close_track_sony(scgp, dp, trackp)
 	register struct	scg_cmd	*scmd = scgp->scmd;
 	int	track = 0;
 
-	fillbytes((caddr_t)scmd, sizeof(*scmd), '\0');
+	if (!is_tao(trackp) && !is_packet(trackp))
+		return (0);
+
+	fillbytes((caddr_t)scmd, sizeof (*scmd), '\0');
 	scmd->flags = SCG_DISRE_ENA;
 	scmd->cdb_len = SC_G1_CDBLEN;
 	scmd->sense_len = CCS_SENSE_LEN;
 	scmd->cdb.g1_cdb.cmd = 0xF0;
 	scmd->cdb.g1_cdb.lun = scg_lun(scgp);
 	g1_cdbaddr(&scmd->cdb.g1_cdb, track);
-/* XXX Padding ??? (bit 0 in addr[0]) */
-	
+/* XXX Padding ??? (bit 0 in addr[0] / CDB[2]) */
+
 	scgp->cmdname = "close_track";
 
 	if (scg_cmd(scgp) < 0)
@@ -286,7 +404,7 @@ close_track_sony(scgp, dp, trackp)
 
 	/*
 	 * Clear the silly "error situation" from Sony dummy write end
-	 * but notify if real errors occurred. 
+	 * but notify if real errors occurred.
 	 */
 	scgp->silent++;
 	if (test_unit_ready(scgp) < 0 && scg_sense_code(scgp) != 0xD4) {
@@ -299,30 +417,32 @@ close_track_sony(scgp, dp, trackp)
 }
 
 LOCAL int
-finalize_sony(scgp, dp, onp, dummy, type, trackp)
+finalize_sony(scgp, dp, trackp)
 	SCSI	*scgp;
 	cdr_t	*dp;
-	int	onp;	/* open next program area */
-	int	dummy;
-	int	type;	/* TOC type 0: CD-DA, 1: CD-ROM, 2: CD-ROM/XA1, 3: CD-ROM/XA2, 4: CDI */
 	track_t	*trackp;
 {
 	register struct	scg_cmd	*scmd = scgp->scmd;
+	int	dummy = track_base(trackp)->tracktype & TOCF_DUMMY;
 
+	if (!is_tao(trackp) && !is_packet(trackp)) {
+		wait_unit_ready(scgp, 240);
+		return (0);
+	}
 	if (dummy) {
 		printf("Fixating is not possible in dummy write mode.\n");
 		return (0);
 	}
-	fillbytes((caddr_t)scmd, sizeof(*scmd), '\0');
+	fillbytes((caddr_t)scmd, sizeof (*scmd), '\0');
 	scmd->flags = SCG_DISRE_ENA;
 	scmd->cdb_len = SC_G1_CDBLEN;
 	scmd->sense_len = CCS_SENSE_LEN;
 	scmd->timeout = 8 * 60;		/* Needs up to 4 minutes */
 	scmd->cdb.g1_cdb.cmd = 0xF1;
 	scmd->cdb.g1_cdb.lun = scg_lun(scgp);
-	scmd->cdb.g1_cdb.count[1] = (onp ? 1 : 0);
-/* XXX Padding ??? (bit 0 in addr[0]) */
-	
+	scmd->cdb.g1_cdb.count[1] = ((track_base(trackp)->tracktype & TOCF_MULTI) ? 1 : 0);
+/* XXX Padding ??? (bit 0 in addr[0] / CDB[2]) */
+
 	scgp->cmdname = "finalize";
 
 	if (scg_cmd(scgp) < 0)
@@ -331,20 +451,47 @@ finalize_sony(scgp, dp, onp, dummy, type, trackp)
 }
 
 LOCAL int
-recover_sony(scgp, track)
+flush_sony(scgp, track)
 	SCSI	*scgp;
 	int	track;
 {
 	register struct	scg_cmd	*scmd = scgp->scmd;
 
-	fillbytes((caddr_t)scmd, sizeof(*scmd), '\0');
+	fillbytes((caddr_t)scmd, sizeof (*scmd), '\0');
+	scmd->flags = SCG_DISRE_ENA;
+	scmd->cdb_len = SC_G1_CDBLEN;
+	scmd->sense_len = CCS_SENSE_LEN;
+	scmd->timeout = 8 * 60;		/* Needs up to 4 minutes */
+	scmd->cdb.g1_cdb.cmd = 0xF2;
+	scmd->cdb.g1_cdb.lun = scg_lun(scgp);
+	scmd->cdb.cmd_cdb[5] = track;
+/* XXX POE ???	   (bit 1 in addr[0] / CDB[2]) */
+/* XXX Padding ??? (bit 0 in addr[0] / CDB[2]) */
+/* XXX Partial flush ??? (CDB[3]) */
+
+	scgp->cmdname = "flush";
+
+	if (scg_cmd(scgp) < 0)
+		return (-1);
+	return (0);
+}
+
+LOCAL int
+recover_sony(scgp, dp, track)
+	SCSI	*scgp;
+	cdr_t	*dp;
+	int	track;
+{
+	register struct	scg_cmd	*scmd = scgp->scmd;
+
+	fillbytes((caddr_t)scmd, sizeof (*scmd), '\0');
 	scmd->flags = SCG_DISRE_ENA;
 	scmd->cdb_len = SC_G1_CDBLEN;
 	scmd->sense_len = CCS_SENSE_LEN;
 	scmd->cdb.g1_cdb.cmd = 0xF6;
 	scmd->cdb.g1_cdb.lun = scg_lun(scgp);
 	scmd->cdb.g1_cdb.addr[3] = track;
-	
+
 	scgp->cmdname = "recover";
 
 	if (scg_cmd(scgp) < 0)
@@ -352,6 +499,31 @@ recover_sony(scgp, track)
 	return (0);
 }
 
+LOCAL int
+set_wr_parameter_sony(scgp, bp, size)
+	SCSI	*scgp;
+	caddr_t	bp;
+	int	size;
+{
+	register struct	scg_cmd	*scmd = scgp->scmd;
+
+	fillbytes((caddr_t)scmd, sizeof (*scmd), '\0');
+	scmd->addr = bp;
+	scmd->size = size;
+	scmd->flags = SCG_DISRE_ENA;
+	scmd->cdb_len = SC_G1_CDBLEN;
+	scmd->sense_len = CCS_SENSE_LEN;
+	scmd->cdb.g1_cdb.cmd = 0xF8;
+	scmd->cdb.g1_cdb.lun = scg_lun(scgp);
+	g1_cdblen(&scmd->cdb.g1_cdb, size);
+
+	scgp->cmdname = "set_write_parameter";
+
+	if (scg_cmd(scgp) < 0)
+		return (-1);
+	return (0);
+}
+
 LOCAL int
 next_wr_addr_sony(scgp, trackp, ap)
 	SCSI	*scgp;
@@ -370,14 +542,14 @@ reserve_track_sony(scgp, len)
 {
 	register struct	scg_cmd	*scmd = scgp->scmd;
 
-	fillbytes((caddr_t)scmd, sizeof(*scmd), '\0');
+	fillbytes((caddr_t)scmd, sizeof (*scmd), '\0');
 	scmd->flags = SCG_DISRE_ENA;
 	scmd->cdb_len = SC_G1_CDBLEN;
 	scmd->sense_len = CCS_SENSE_LEN;
 	scmd->cdb.g1_cdb.cmd = 0xF3;
 	scmd->cdb.g1_cdb.lun = scg_lun(scgp);
 	i_to_4_byte(&scmd->cdb.g1_cdb.addr[3], len);
-	
+
 	scgp->cmdname = "reserve_track";
 
 	if (scg_cmd(scgp) < 0)
@@ -385,7 +557,16 @@ reserve_track_sony(scgp, len)
 	return (0);
 }
 
-#define	IS(what,flag)	printf("  Is %s%s\n", flag?"":"not ",what);
+LOCAL int
+init_sony(scgp, dp)
+	SCSI	*scgp;
+	cdr_t	*dp;
+{
+	return (speed_select_sony(scgp, dp, NULL));
+}
+
+
+#define	IS(what, flag)	printf("  Is %s%s\n", flag?"":"not ", what);
 
 LOCAL int
 getdisktype_sony(scgp, dp)
@@ -404,7 +585,7 @@ getdisktype_sony(scgp, dp)
 	dummy = get_page22_sony(scgp, mode);
 	if (dummy >= 0) {
 		mp = (struct scsi_mode_page_header *)
-			(mode + sizeof(struct scsi_mode_header) +
+			(mode + sizeof (struct scsi_mode_header) +
 			((struct scsi_mode_header *)mode)->blockdesc_len);
 
 		xp = (struct sony_924_mode_page_22 *)mp;
@@ -523,16 +704,16 @@ int	sony_speeds[] = {
 };
 
 LOCAL int
-speed_select_sony(scgp, dp, speedp, dummy)
+speed_select_sony(scgp, dp, speedp)
 	SCSI	*scgp;
 	cdr_t	*dp;
 	int	*speedp;
-	int	dummy;
 {
 	struct cdd_52x_mode_data md;
 	int	count;
 	int	err;
 	int	speed = 1;
+	BOOL	dummy = (dp->cdr_cmdflags & F_DUMMY) != 0;
 
 	if (speedp) {
 		speed = *speedp;
@@ -540,15 +721,21 @@ speed_select_sony(scgp, dp, speedp, dummy)
 			return (-1);
 	}
 
-	fillbytes((caddr_t)&md, sizeof(md), '\0');
+	fillbytes((caddr_t)&md, sizeof (md), '\0');
 
-	count  = sizeof(struct scsi_mode_header) +
-		sizeof(struct sony_924_mode_page_20);
+	count  = sizeof (struct scsi_mode_header) +
+		sizeof (struct sony_924_mode_page_20);
 
 	md.pagex.page_s20.p_code = 0x20;
 	md.pagex.page_s20.p_len =  0x06;
 	md.pagex.page_s20.speudo = dummy?1:0;
-	
+
+	/*
+	 * Set Cue sheet option. This is documented for the 924 and
+	 * seems to be supported for the 948 too.
+	 */
+	md.pagex.page_s20.cue_sheet_opt = 0x03;
+
 	err = mode_select(scgp, (Uchar *)&md, count, 0, 1);
 	if (err < 0)
 		return (err);
@@ -556,15 +743,15 @@ speed_select_sony(scgp, dp, speedp, dummy)
 	if (speedp == 0)
 		return (0);
 
-	fillbytes((caddr_t)&md, sizeof(md), '\0');
+	fillbytes((caddr_t)&md, sizeof (md), '\0');
 
-	count  = sizeof(struct scsi_mode_header) +
-		sizeof(struct sony_924_mode_page_31);
+	count  = sizeof (struct scsi_mode_header) +
+		sizeof (struct sony_924_mode_page_31);
 
 	md.pagex.page_s31.p_code = 0x31;
 	md.pagex.page_s31.p_len =  0x02;
 	md.pagex.page_s31.speed = sony_speeds[speed];
-	
+
 	return (mode_select(scgp, (Uchar *)&md, count, 0, 1));
 }
 
@@ -582,7 +769,7 @@ next_writable_address_sony(scgp, ap, track, sectype, tracktype)
 	int			page = 0x23;
 	struct sony_924_mode_page_23	*xp;
 
-	fillbytes((caddr_t)mode, sizeof(mode), '\0');
+	fillbytes((caddr_t)mode, sizeof (mode), '\0');
 
 	inc_verbose();
 	if (!get_mode_params(scgp, page, "CD track information",
@@ -595,7 +782,7 @@ next_writable_address_sony(scgp, ap, track, sectype, tracktype)
 		return (-1);
 
 	mp = (struct scsi_mode_page_header *)
-		(mode + sizeof(struct scsi_mode_header) +
+		(mode + sizeof (struct scsi_mode_header) +
 		((struct scsi_mode_header *)mode)->blockdesc_len);
 
 
@@ -624,10 +811,10 @@ new_track_sony(scgp, track, sectype, tracktype)
 	struct sony_924_mode_page_23	*xp;
 	int	i;
 
-	fillbytes((caddr_t)mode, sizeof(mode), '\0');
+	fillbytes((caddr_t)mode, sizeof (mode), '\0');
 	get_page22_sony(scgp, mode);
 
-	fillbytes((caddr_t)mode, sizeof(mode), '\0');
+	fillbytes((caddr_t)mode, sizeof (mode), '\0');
 
 	inc_verbose();
 	if (!get_mode_params(scgp, page, "CD track information",
@@ -640,7 +827,7 @@ new_track_sony(scgp, track, sectype, tracktype)
 		return (-1);
 
 	mp = (struct scsi_mode_page_header *)
-		(mode + sizeof(struct scsi_mode_header) +
+		(mode + sizeof (struct scsi_mode_header) +
 		((struct scsi_mode_header *)mode)->blockdesc_len);
 
 
@@ -689,10 +876,28 @@ open_track_sony(scgp, dp, trackp)
 	cdr_t	*dp;
 	track_t *trackp;
 {
+	if (!is_tao(trackp) && !is_packet(trackp)) {
+		if (trackp->pregapsize > 0 && (trackp->flags & TI_PREGAP) == 0) {
+			if (lverbose) {
+				printf("Writing pregap for track %d at %ld\n",
+					(int)trackp->trackno,
+					trackp->trackstart-trackp->pregapsize);
+			}
+			/*
+			 * XXX Do we need to check isecsize too?
+			 */
+			pad_track(scgp, dp, trackp,
+				trackp->trackstart-trackp->pregapsize,
+				(Llong)trackp->pregapsize*trackp->secsize,
+					FALSE, 0);
+		}
+		return (0);
+	}
+
 	if (select_secsize(scgp, trackp->secsize) < 0)
 		return (-1);
 
-	if (new_track_sony(scgp, trackp->trackno, trackp->sectype, trackp->tracktype) < 0)
+	if (new_track_sony(scgp, trackp->trackno, trackp->sectype, trackp->tracktype & TOC_MASK) < 0)
 		return (-1);
 
 	if (write_track_sony(scgp, 0L, trackp->sectype) < 0)
@@ -702,12 +907,10 @@ open_track_sony(scgp, dp, trackp)
 }
 
 LOCAL int
-open_session_sony(scgp, dp, trackp, toctype, multi)
+open_session_sony(scgp, dp, trackp)
 	SCSI	*scgp;
 	cdr_t	*dp;
 	track_t	*trackp;
-	int	toctype;
-	int	multi;
 {
 	struct	scsi_mode_page_header *mp;
 	char			mode[256];
@@ -715,18 +918,37 @@ open_session_sony(scgp, dp, trackp, toctype, multi)
 	int	len = 0x30;
 	struct sony_924_mode_page_22	*xp;
 
-	fillbytes((caddr_t)mode, sizeof(mode), '\0');
+	fillbytes((caddr_t)mode, sizeof (mode), '\0');
 
 	if ((len = get_page22_sony(scgp, mode)) < 0)
 		return (-1);
 
 	mp = (struct scsi_mode_page_header *)
-		(mode + sizeof(struct scsi_mode_header) +
+		(mode + sizeof (struct scsi_mode_header) +
 		((struct scsi_mode_header *)mode)->blockdesc_len);
 
 	xp = (struct sony_924_mode_page_22 *)mp;
 
-	xp->disk_type = toc2sess[toctype & TOC_MASK];
+	xp->disk_type = toc2sess[track_base(trackp)->tracktype & TOC_MASK];
+
+	if (is_tao(track_base(trackp))) {
+#ifdef	__needed__
+		if ((track_base(trackp)->tracktype & TOC_MASK) == TOC_DA)
+			xp->disk_style = 0x80;
+		else
+			xp->disk_style = 0xC0;
+#endif
+	} else if (is_sao(track_base(trackp))) {
+		/*
+		 * We may only change this value if the disk is empty.
+		 * i.e. when disk_status & 0xC0 == 0x00
+		 */
+		if ((xp->disk_status & 0xC0) != 0) {
+			if (xp->disk_style != 0x00)
+				errmsgno(EX_BAD, "Cannot change disk stile for recorded disk.\n");
+		}
+		xp->disk_style = 0x00;
+	}
 
 	((struct scsi_modesel_header *)mode)->sense_data_len	= 0;
 	((struct scsi_modesel_header *)mode)->res2		= 0;
@@ -741,9 +963,21 @@ open_session_sony(scgp, dp, trackp, toctype, multi)
 	if (mode_select(scgp, (Uchar *)mode, len, 0, scgp->inq->data_format >= 2) < 0) {
 		return (-1);
 	}
+/*
+ * XXX set write parameter fr SAO mit Multi Session (948 only?)
+ * XXX set_wr_parameter_sony(scgp, bp, size);
+ */
 	return (0);
 }
 
+LOCAL int
+abort_session_sony(scgp, dp)
+	SCSI	*scgp;
+	cdr_t	*dp;
+{
+	return (discontinue_sony(scgp));
+}
+
 LOCAL int
 get_page22_sony(scgp, mode)
 	SCSI	*scgp;
@@ -754,7 +988,7 @@ get_page22_sony(scgp, mode)
 	int	page = 0x22;
 	struct sony_924_mode_page_22	*xp;
 
-	fillbytes((caddr_t)mode, sizeof(mode), '\0');
+	fillbytes((caddr_t)mode, sizeof (mode), '\0');
 
 	inc_verbose();
 	if (!get_mode_params(scgp, page, "CD disk information",
@@ -767,7 +1001,7 @@ get_page22_sony(scgp, mode)
 		return (-1);
 
 	mp = (struct scsi_mode_page_header *)
-		(mode + sizeof(struct scsi_mode_header) +
+		(mode + sizeof (struct scsi_mode_header) +
 		((struct scsi_mode_header *)mode)->blockdesc_len);
 
 	xp = (struct sony_924_mode_page_22 *)mp;
@@ -778,6 +1012,266 @@ get_page22_sony(scgp, mode)
 	return (len);
 }
 
+/*--------------------------------------------------------------------------*/
+
+LOCAL Uchar	db2df[] = {
+	0x01,			/*  0 2352 bytes of raw data			*/
+	0xFF,			/*  1 2368 bytes (raw data + P/Q Subchannel)	*/
+	0xFF,			/*  2 2448 bytes (raw data + P-W Subchannel)	*/
+	0xFF,			/*  3 2448 bytes (raw data + P-W raw Subchannel)*/
+	0xFF,			/*  4 -    Reserved				*/
+	0xFF,			/*  5 -    Reserved				*/
+	0xFF,			/*  6 -    Reserved				*/
+	0xFF,			/*  7 -    Vendor specific			*/
+	0x11,			/*  8 2048 bytes Mode 1 (ISO/IEC 10149)		*/
+	0xFF,			/*  9 2336 bytes Mode 2 (ISO/IEC 10149)		*/
+	0xFF,			/* 10 2048 bytes Mode 2! (CD-ROM XA form 1)	*/
+	0xFF,			/* 11 2056 bytes Mode 2 (CD-ROM XA form 1)	*/
+	0xFF,			/* 12 2324 bytes Mode 2 (CD-ROM XA form 2)	*/
+	0xFF,			/* 13 2332 bytes Mode 2 (CD-ROM XA 1/2+subhdr)	*/
+	0xFF,			/* 14 -    Reserved				*/
+	0xFF,			/* 15 -    Vendor specific			*/
+};
+
+LOCAL int
+gen_cue_sony(trackp, vcuep, needgap)
+	track_t	*trackp;
+	void	*vcuep;
+	BOOL	needgap;
+{
+	int	tracks = trackp->tracks;
+	int	i;
+	struct sony_cue	**cuep = vcuep;
+	struct sony_cue	*cue;
+	struct sony_cue	*cp;
+	int	ncue = 0;
+	int	icue = 0;
+	int	pgsize;
+	msf_t	m;
+	int	ctl;
+	int	df;
+	int	scms;
+
+	cue = malloc(1);
+
+	for (i = 0; i <= tracks; i++) {
+		ctl = (st2mode[trackp[i].sectype & ST_MASK]) << 4;
+		if (is_copy(&trackp[i]))
+			ctl |= TM_ALLOW_COPY << 4;
+		df = db2df[trackp[i].dbtype & 0x0F];
+
+#ifdef	__supported__
+		if (trackp[i].isrc) {	/* MCN or ISRC */
+			ncue += 2;
+			cue = realloc(cue, ncue * sizeof (*cue));
+			cp = &cue[icue++];
+			if (i == 0) {
+				cp->cs_ctladr = 0x02;
+				movebytes(&trackp[i].isrc[0], &cp->cs_tno, 7);
+				cp = &cue[icue++];
+				cp->cs_ctladr = 0x02;
+				movebytes(&trackp[i].isrc[7], &cp->cs_tno, 7);
+			} else {
+				cp->cs_ctladr = 0x03;
+				cp->cs_tno = i;
+				movebytes(&trackp[i].isrc[0], &cp->cs_index, 6);
+				cp = &cue[icue++];
+				cp->cs_ctladr = 0x03;
+				cp->cs_tno = i;
+				movebytes(&trackp[i].isrc[6], &cp->cs_index, 6);
+			}
+		}
+#endif
+		if (i == 0) {	/* Lead in */
+			df &= ~7;
+			if (trackp[0].flags & TI_TEXT)	/* CD-Text in Lead-in*/
+				df |= 0xC0;
+			lba_to_msf(-150, &m);
+			cue = realloc(cue, ++ncue * sizeof (*cue));
+			cp = &cue[icue++];
+			fillcue(cp, ctl|0x01, i, 0, df, 0, &m);
+		} else {
+			scms = 0;
+
+			if (is_scms(&trackp[i]))
+				scms = 0x80;
+			pgsize = trackp[i].pregapsize;
+			if (pgsize == 0 && needgap)
+				pgsize++;
+			lba_to_msf(trackp[i].trackstart-pgsize, &m);
+			cue = realloc(cue, ++ncue * sizeof (*cue));
+			cp = &cue[icue++];
+			fillcue(cp, ctl|0x01, i, 0, df, scms, &m);
+
+			if (trackp[i].nindex == 1) {
+				lba_to_msf(trackp[i].trackstart, &m);
+				cue = realloc(cue, ++ncue * sizeof (*cue));
+				cp = &cue[icue++];
+				fillcue(cp, ctl|0x01, i, 1, df, scms, &m);
+			} else {
+				int	idx;
+				long	*idxlist;
+
+				ncue += trackp[i].nindex;
+				idxlist = trackp[i].tindex;
+				cue = realloc(cue, ncue * sizeof (*cue));
+
+				for (idx = 1; idx <= trackp[i].nindex; idx++) {
+					lba_to_msf(trackp[i].trackstart + idxlist[idx], &m);
+					cp = &cue[icue++];
+					fillcue(cp, ctl|0x01, i, idx, df, scms, &m);
+				}
+			}
+		}
+	}
+	/* Lead out */
+	ctl = (st2mode[trackp[tracks+1].sectype & ST_MASK]) << 4;
+	df = db2df[trackp[tracks+1].dbtype & 0x0F];
+	df &= ~7;
+	lba_to_msf(trackp[tracks+1].trackstart, &m);
+	cue = realloc(cue, ++ncue * sizeof (*cue));
+	cp = &cue[icue++];
+	fillcue(cp, ctl|0x01, 0xAA, 1, df, 0, &m);
+
+	if (lverbose > 1) {
+		for (i = 0; i < ncue; i++) {
+			scg_prbytes("", (Uchar *)&cue[i], 8);
+		}
+	}
+	if (cuep)
+		*cuep = cue;
+	else
+		free(cue);
+	return (ncue);
+}
+
+
+LOCAL void
+fillcue(cp, ca, tno, idx, dataform, scms, mp)
+	struct sony_cue *cp;	/* The target cue entry		*/
+	int	ca;		/* Control/adr for this entry	*/
+	int	tno;		/* Track number for this entry	*/
+	int	idx;		/* Index for this entry		*/
+	int	dataform;	/* Data format for this entry	*/
+	int	scms;		/* Serial copy management	*/
+	msf_t	*mp;		/* MSF value for this entry	*/
+{
+	cp->cs_ctladr = ca;
+	if (tno <= 99)
+		cp->cs_tno = to_bcd(tno);
+	else
+		cp->cs_tno = tno;
+	cp->cs_index = to_bcd(idx);
+	cp->cs_dataform = dataform;
+	cp->cs_zero = scms;
+	cp->cs_min = to_bcd(mp->msf_min);
+	cp->cs_sec = to_bcd(mp->msf_sec);
+	cp->cs_frame = to_bcd(mp->msf_frame);
+}
+
+LOCAL int
+send_cue_sony(scgp, dp, trackp)
+	SCSI	*scgp;
+	cdr_t	*dp;
+	track_t	*trackp;
+{
+	struct sony_cue *cp;
+	int		ncue;
+	int		ret;
+	Uint		i;
+	struct timeval starttime;
+	struct timeval stoptime;
+	int		disktype;
+
+	disktype = toc2sess[track_base(trackp)->tracktype & TOC_MASK];
+
+	for (i = 1; i <= trackp->tracks; i++) {
+		if (trackp[i].tracksize < (tsize_t)0) {
+			errmsgno(EX_BAD, "Track %d has unknown length.\n", i);
+			return (-1);
+		}
+	}
+	ncue = (*dp->cdr_gen_cue)(trackp, &cp, FALSE);
+
+	starttime.tv_sec = 0;
+	starttime.tv_usec = 0;
+	stoptime = starttime;
+	gettimeofday(&starttime, (struct timezone *)0);
+
+	scgp->silent++;
+	ret  = write_start_sony(scgp, (caddr_t)cp, ncue*8);
+	scgp->silent--;
+	free(cp);
+	if (ret < 0) {
+		errmsgno(EX_BAD, "CUE sheet not accepted. Retrying with minimum pregapsize = 1.\n");
+		ncue = (*dp->cdr_gen_cue)(trackp, &cp, TRUE);
+		ret  = write_start_sony(scgp, (caddr_t)cp, ncue*8);
+		free(cp);
+	}
+	if (ret >= 0 && lverbose) {
+		gettimeofday(&stoptime, (struct timezone *)0);
+		prtimediff("Write Lead-in time: ", &starttime, &stoptime);
+	}
+	return (ret);
+}
+
+LOCAL int
+write_leadin_sony(scgp, dp, trackp)
+	SCSI	*scgp;
+	cdr_t	*dp;
+	track_t *trackp;
+{
+	Uint	i;
+	long	startsec = 0L;
+
+/*	if (flags & F_SAO) {*/
+	if (wm_base(dp->cdr_dstat->ds_wrmode) == WM_SAO) {
+		if (debug || lverbose) {
+			printf("Sending CUE sheet...\n");
+			flush();
+		}
+		if (trackp[0].flags & TI_TEXT) {
+			if (dp->cdr_speeddef != 4) {
+				errmsgno(EX_BAD,
+				"The CDU-924 does not support CD-Text, disabling.\n");
+
+				trackp[0].flags &= ~TI_TEXT;
+			}
+		}
+		if ((*dp->cdr_send_cue)(scgp, dp, trackp) < 0) {
+			errmsgno(EX_BAD, "Cannot send CUE sheet.\n");
+			return (-1);
+		}
+
+		if (trackp[0].flags & TI_TEXT) {
+			startsec = dp->cdr_dstat->ds_first_leadin;
+			printf("SAO startsec: %ld\n", startsec);
+		} else {
+			startsec = -150;
+		}
+		if (debug)
+			printf("SAO startsec: %ld\n", startsec);
+
+		if (trackp[0].flags & TI_TEXT) {
+			if (startsec > 0) {
+				errmsgno(EX_BAD, "CD-Text must be in first session.\n");
+				return (-1);
+			}
+			if (debug || lverbose)
+				printf("Writing lead-in...\n");
+			if (write_cdtext(scgp, dp, startsec) < 0)
+				return (-1);
+
+			dp->cdr_dstat->ds_cdrflags |= RF_LEADIN;
+		} else for (i = 1; i <= trackp->tracks; i++) {
+			trackp[i].trackstart += startsec +150;
+		}
+	}
+	return (0);
+}
+
+/*--------------------------------------------------------------------------*/
+
 static const char *sd_cdu_924_error_str[] = {
 
 	"\200\000write complete",				/* 80 00 */
@@ -788,7 +1282,7 @@ static const char *sd_cdu_924_error_str[] = {
 
 	"\266\000media load mechanism failed",			/* B6 00 */
 	"\271\000audio play operation aborted",			/* B9 00 */
-	"\277\000buffer overflow for read all subcodes command",/* BF 00 */
+	"\277\000buffer overflow for read all subcodes command", /* BF 00 */
 	"\300\000unrecordable disk",				/* C0 00 */
 	"\301\000illegal track status",				/* C1 00 */
 	"\302\000reserved track present",			/* C2 00 */
@@ -826,6 +1320,11 @@ sony_attach(scgp, dp)
 	SCSI	*scgp;
 	cdr_t	*dp;
 {
+	if (scgp->inq != NULL) {
+		if (strbeg("CD-R   CDU94", scgp->inq->prod_ident)) {
+			dp->cdr_speeddef = 4;
+		}
+	}
 	scg_setnonstderrs(scgp, sd_cdu_924_error_str);
 	return (0);
 }
@@ -889,15 +1388,15 @@ buf_cap_sony(scgp, sp, fp)
 	int	per;
 	register struct	scg_cmd	*scmd = scgp->scmd;
 
-	fillbytes((caddr_t)scmd, sizeof(*scmd), '\0');
+	fillbytes((caddr_t)scmd, sizeof (*scmd), '\0');
 	scmd->addr = (caddr_t)resp;
-	scmd->size = sizeof(resp);
+	scmd->size = sizeof (resp);
 	scmd->flags = SCG_RECV_DATA|SCG_DISRE_ENA;
 	scmd->cdb_len = SC_G1_CDBLEN;
 	scmd->sense_len = CCS_SENSE_LEN;
 	scmd->cdb.g1_cdb.cmd = 0xEC;		/* Read buffer cap */
 	scmd->cdb.g1_cdb.lun = scg_lun(scgp);
-	
+
 	scgp->cmdname = "read buffer cap sony";
 
 	if (scg_cmd(scgp) < 0)
@@ -909,7 +1408,7 @@ buf_cap_sony(scgp, sp, fp)
 		*sp = bufsize;
 	if (fp)
 		*fp = freespace;
-	
+
 	if (scgp->verbose || (sp == 0 && fp == 0))
 		printf("BFree: %ld K BSize: %ld K\n", freespace >> 10, bufsize >> 10);
 
