@@ -1,7 +1,7 @@
-/* @(#)hfs.c	1.8 02/09/28 joerg */
+/* @(#)hfs.c	1.9 04/06/17 joerg */
 #ifndef lint
 static	char sccsid[] =
-	"@(#)hfs.c	1.8 02/09/28 joerg";
+	"@(#)hfs.c	1.9 04/06/17 joerg";
 #endif
 /*
  * hfsutils - tools for reading and writing Macintosh HFS volumes
@@ -1140,7 +1140,7 @@ int hfs_readdir(dir, ent)
       return -1;
     }
 
-  while (1)
+  for (;;)
     {
       ++dir->n.rnum;
 
