@@ -1,4 +1,4 @@
-/* @(#)fdup64.c	1.5 00/12/28 Copyright 2000 J. Schilling */
+/* @(#)fdup64.c	1.6 04/08/08 Copyright 2000 J. Schilling */
 /*
  *	Copyright (c) 2000 J. Schilling
  */
@@ -13,14 +13,14 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License along with
+ * this program; see the file COPYING.  If not, write to the Free Software
+ * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #define	USE_LARGEFILES
 
-#include "io.h"
+#include "schilyio.h"
 
 /*#if	_LFS_LARGEFILE*/
 #ifdef	HAVE_LARGEFILES
