@@ -1,13 +1,13 @@
-/* @(#)dvd_file.c	1.2 02/12/23 joerg */
+/* @(#)dvd_file.c	1.3 04/03/04 joerg */
 #ifndef lint
 static	char sccsid[] =
-	"@(#)dvd_file.c	1.2 02/12/23 joerg";
+	"@(#)dvd_file.c	1.3 04/03/04 joerg";
 #endif
-/* 
- * DVD_VIDEO code 
+/*
+ * DVD_VIDEO code
  *  Copyright (c) 2002 Olaf Beck - olaf_sc@yahoo.com
- *		       Jrg Schilling <schilling@fokus.gmd.de>
- *		       (making the code portable)
+ *			Jrg Schilling <schilling@fokus.gmd.de>
+ *			(making the code portable)
  */
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -36,16 +36,16 @@ static	char sccsid[] =
 
 LOCAL	void	bsort		__PR((int sector[], int title[], int size));
 LOCAL	void	uniq		__PR((int sector[], int title[],
-					int title_sets_array[], 
+					int title_sets_array[],
 					int sector_sets_array[], int titles));
-LOCAL	void	DVDFreeFileSetArrays __PR((int * sector, int * title, int * title_sets_array, 
+LOCAL	void	DVDFreeFileSetArrays __PR((int * sector, int * title, int * title_sets_array,
 					int * sector_sets_array));
 EXPORT	void	DVDFreeFileSet	__PR((title_set_info_t * title_set_info));
 EXPORT	title_set_info_t * DVDGetFileSet __PR((char * dvd));
 EXPORT	int	DVDGetFilePad	__PR((title_set_info_t * title_set_info, char * name));
 
 
-LOCAL void 
+LOCAL void
 bsort(sector, title, size)
 	int	sector[];
 	int	title[];
@@ -56,8 +56,8 @@ bsort(sector, title, size)
 	int	i;
 	int	j;
 
-	for (i=0; i < size; i++) {
-		for (j=0; j < size; j++) {
+	for (i = 0; i < size; i++) {
+		for (j = 0; j < size; j++) {
 			if (sector[i] < sector[j]) {
 				temp_sector = sector[i];
 				temp_title = title[i];
@@ -83,7 +83,7 @@ uniq(sector, title, title_sets_array, sector_sets_array, titles)
 	int	j;
 
 
-	for (i=0, j=0; j < titles;) {
+	for (i = 0, j = 0; j < titles; ) {
 		if (sector[j] != sector[j+1]) {
 			title_sets_array[i]  = title[j];
 			sector_sets_array[i] = sector[j];
@@ -104,7 +104,7 @@ uniq(sector, title, title_sets_array, sector_sets_array, titles)
 
 }
 
-LOCAL void 
+LOCAL void
 DVDFreeFileSetArrays(sector, title, title_sets_array, sector_sets_array)
 	int	* sector;
 	int	* title;
@@ -117,7 +117,7 @@ DVDFreeFileSetArrays(sector, title, title_sets_array, sector_sets_array)
 	free(sector_sets_array);
 }
 
-EXPORT void 
+EXPORT void
 DVDFreeFileSet(title_set_info)
 	title_set_info_t * title_set_info;
 {
@@ -130,11 +130,11 @@ DVDGetFileSet(dvd)
 	char * dvd;
 {
 	/*
-	 * TODO  Fix close of files if 
-	 *       we error out
-	 *       We also assume that all
-	 *       DVD files are of valid
-	 *       size i.e. file%2048 == 0
+	 * TODO  Fix close of files if
+	 *	we error out
+	 *	We also assume that all
+	 *	DVD files are of valid
+	 *	size i.e. file%2048 == 0
 	 */
 
 	/* title interation */
@@ -146,15 +146,15 @@ DVDGetFileSet(dvd)
 	/* DVD file structures */
 	dvd_reader_t *	_dvd = NULL;
 
-	ifo_handle_t *	vmg_ifo=NULL;
-	ifo_handle_t *	vts_ifo=NULL;
+	ifo_handle_t *	vmg_ifo = NULL;
+	ifo_handle_t *	vts_ifo = NULL;
 
-	dvd_file_t   *	vmg_vob_file=NULL;
-	dvd_file_t   *	vmg_ifo_file=NULL;
+	dvd_file_t   *	vmg_vob_file = NULL;
+	dvd_file_t   *	vmg_ifo_file = NULL;
 
-	dvd_file_t   *	vts_ifo_file=NULL;
-	dvd_file_t   *	vts_menu_file=NULL;
-	dvd_file_t   * 	vts_title_file=NULL;
+	dvd_file_t   *	vts_ifo_file = NULL;
+	dvd_file_t   *	vts_menu_file = NULL;
+	dvd_file_t   * 	vts_title_file = NULL;
 
 	/* The sizes it self of each file */
 	int		ifo;
@@ -178,9 +178,9 @@ DVDGetFileSet(dvd)
 	/* Temporary mount point - to be used later */
 	char		mountpoint[PATH_MAX + 1];
 
-	strncpy(mountpoint, dvd, sizeof(mountpoint));
-	mountpoint[sizeof(mountpoint)-1] = '\0';
-	
+	strncpy(mountpoint, dvd, sizeof (mountpoint));
+	mountpoint[sizeof (mountpoint)-1] = '\0';
+
 
 	_dvd = DVDOpen(dvd);
 	if (!_dvd) {
@@ -203,7 +203,7 @@ DVDGetFileSet(dvd)
 
 	/* Check mount point */
 
-	snprintf(temppoint, sizeof(temppoint),
+	snprintf(temppoint, sizeof (temppoint),
 				"%s/VIDEO_TS/VIDEO_TS.IFO", mountpoint);
 
 
@@ -223,14 +223,14 @@ DVDGetFileSet(dvd)
 	title_sets = vmg_ifo->vmgi_mat->vmg_nr_of_title_sets;
 	titles = vmg_ifo->tt_srpt->nr_of_srpts;
 
-	sector = e_malloc(titles * sizeof(int));
-	memset(sector, 0, titles * sizeof(int));
-	title = e_malloc(titles * sizeof(int));
-	title_sets_array = e_malloc(title_sets * sizeof(int));
-	sector_sets_array = e_malloc(title_sets * sizeof(int));
-	title_set_info = (title_set_info_t *)e_malloc(sizeof(title_set_info_t));
-	title_set_info->title_set = (title_set_t *)e_malloc((title_sets + 1) * 
-							sizeof(title_set_t));
+	sector = e_malloc(titles * sizeof (int));
+	memset(sector, 0, titles * sizeof (int));
+	title = e_malloc(titles * sizeof (int));
+	title_sets_array = e_malloc(title_sets * sizeof (int));
+	sector_sets_array = e_malloc(title_sets * sizeof (int));
+	title_set_info = (title_set_info_t *)e_malloc(sizeof (title_set_info_t));
+	title_set_info->title_set = (title_set_t *)e_malloc((title_sets + 1) *
+							sizeof (title_set_t));
 
 	title_set_info->num_titles = title_sets;
 
@@ -238,9 +238,9 @@ DVDGetFileSet(dvd)
 	/* Fill and sort the arrays for titles*/
 
 	if (titles >= 1) {
-		for (counter=0; counter < titles; counter++) {
-			sector[counter]=vmg_ifo->tt_srpt->title[counter].title_set_sector;
-			title[counter]=counter + 1;
+		for (counter = 0; counter < titles; counter++) {
+			sector[counter] = vmg_ifo->tt_srpt->title[counter].title_set_sector;
+			title[counter]  = counter + 1;
 		}
 	}
 
@@ -249,7 +249,7 @@ DVDGetFileSet(dvd)
 
 
 	/*
-	 * Since title sets and titles are not the same we will need to sort 
+	 * Since title sets and titles are not the same we will need to sort
 	 * out "bogus" titles
 	 */
 
@@ -264,19 +264,19 @@ DVDGetFileSet(dvd)
 
 	vmg_ifo_file = DVDOpenFile(_dvd, 0, DVD_READ_INFO_FILE);
 
-	if ((vmg_vob_file == 0) && vmg_ifo->vmgi_mat->vmg_last_sector + 1 
+	if ((vmg_vob_file == 0) && vmg_ifo->vmgi_mat->vmg_last_sector + 1
 			< 2 * DVDFileSize(vmg_ifo_file)) {
 #ifdef	USE_LIBSCHILY
 		errmsgno(EX_BAD, "IFO is not of correct size aborting\n");
 #else
 		fprintf(stderr, "IFO is not of correct size aborting\n");
 #endif
-		DVDFreeFileSetArrays(sector, title, title_sets_array, 
+		DVDFreeFileSetArrays(sector, title, title_sets_array,
 					sector_sets_array);
 		DVDFreeFileSet(title_set_info);
 		return (0);
-	} else if ((vmg_vob_file != 0) && (vmg_ifo->vmgi_mat->vmg_last_sector 
-		    + 1  < 2 * DVDFileSize(vmg_ifo_file) + 
+	} else if ((vmg_vob_file != 0) && (vmg_ifo->vmgi_mat->vmg_last_sector
+		    + 1  < 2 * DVDFileSize(vmg_ifo_file) +
 		    DVDFileSize(vmg_vob_file))) {
 #ifdef	USE_LIBSCHILY
 		errmsgno(EX_BAD, "Either VIDEO_TS.IFO or VIDEO_TS.VOB is not of correct size");
@@ -291,19 +291,19 @@ DVDGetFileSet(dvd)
 
 	/* Find the actuall right size of VIDEO_TS.IFO */
 	if (vmg_vob_file == 0) {
-		if (vmg_ifo->vmgi_mat->vmg_last_sector + 1 > 2 
+		if (vmg_ifo->vmgi_mat->vmg_last_sector + 1 > 2
 				*  DVDFileSize(vmg_ifo_file)) {
-			ifo=vmg_ifo->vmgi_mat->vmg_last_sector 
-			    - DVDFileSize(vmg_ifo_file) + 1;
+			ifo = vmg_ifo->vmgi_mat->vmg_last_sector
+				- DVDFileSize(vmg_ifo_file) + 1;
 		} else {
-			ifo=vmg_ifo->vmgi_mat->vmgi_last_sector + 1;
+			ifo = vmg_ifo->vmgi_mat->vmgi_last_sector + 1;
 		}
 	} else {
-		if (vmg_ifo->vmgi_mat->vmgi_last_sector + 1 
+		if (vmg_ifo->vmgi_mat->vmgi_last_sector + 1
 				< vmg_ifo->vmgi_mat->vmgm_vobs) {
-			ifo=vmg_ifo->vmgi_mat->vmgm_vobs;
+			ifo = vmg_ifo->vmgi_mat->vmgm_vobs;
 		} else {
-			ifo=vmg_ifo->vmgi_mat->vmgi_last_sector + 1;
+			ifo = vmg_ifo->vmgi_mat->vmgi_last_sector + 1;
 		}
 	}
 
@@ -313,18 +313,18 @@ DVDGetFileSet(dvd)
 
 	/* Find the actuall right size of VIDEO_TS.VOB */
 	if (vmg_vob_file != 0) {
-		if (ifo + DVDFileSize(vmg_ifo_file) + 
-		     DVDFileSize(vmg_vob_file) - 1 < 
-		     vmg_ifo->vmgi_mat->vmg_last_sector) {
+		if (ifo + DVDFileSize(vmg_ifo_file) +
+		    DVDFileSize(vmg_vob_file) - 1 <
+		    vmg_ifo->vmgi_mat->vmg_last_sector) {
 				menu_vob = vmg_ifo->vmgi_mat->vmg_last_sector -
-					   ifo - DVDFileSize(vmg_ifo_file) + 1;
+						ifo - DVDFileSize(vmg_ifo_file) + 1;
 		} else {
-			menu_vob = vmg_ifo->vmgi_mat->vmg_last_sector 
+			menu_vob = vmg_ifo->vmgi_mat->vmg_last_sector
 			- ifo - DVDFileSize(vmg_ifo_file) + 1;
 		}
 
-		snprintf(temppoint, sizeof(temppoint),
-				"%s/VIDEO_TS/VIDEO_TS.VOB", mountpoint);	
+		snprintf(temppoint, sizeof (temppoint),
+				"%s/VIDEO_TS/VIDEO_TS.VOB", mountpoint);
 		if (stat(temppoint, &fileinfo) < 0) {
 #ifdef	USE_LIBSCHILY
 			errmsg("calc: Can't stat %s\n", temppoint);
@@ -332,14 +332,14 @@ DVDGetFileSet(dvd)
 			fprintf(stderr, "calc: Can't stat %s\n", temppoint);
 			perror("");
 #endif
-			DVDFreeFileSetArrays(sector, title, title_sets_array, 
+			DVDFreeFileSetArrays(sector, title, title_sets_array,
 						sector_sets_array);
 			DVDFreeFileSet(title_set_info);
 			return (0);
 		}
 
 		title_set_info->title_set[0].realsize_menu = fileinfo.st_size;
-		title_set_info->title_set[0].pad_menu = menu_vob - 
+		title_set_info->title_set[0].pad_menu = menu_vob -
 						DVDFileSize(vmg_vob_file);
 		title_set_info->title_set[0].size_menu = menu_vob * 2048;
 		DVDCloseFile(vmg_vob_file);
@@ -359,10 +359,10 @@ DVDGetFileSet(dvd)
 		bup = vmg_ifo->vmgi_mat->vmg_last_sector + 1 - menu_vob - ifo;
 	}
 
-	/* Never trust the BUP file - use a copy of the IFO */	
-	snprintf(temppoint, sizeof(temppoint),
+	/* Never trust the BUP file - use a copy of the IFO */
+	snprintf(temppoint, sizeof (temppoint),
 				"%s/VIDEO_TS/VIDEO_TS.IFO", mountpoint);
-	
+
 	if (stat(temppoint, &fileinfo) < 0) {
 #ifdef	USE_LIBSCHILY
 		errmsg("calc: Can't stat %s\n", temppoint);
@@ -370,55 +370,54 @@ DVDGetFileSet(dvd)
 		fprintf(stderr, "calc: Can't stat %s\n", temppoint);
 		perror("");
 #endif
-		DVDFreeFileSetArrays(sector, title, title_sets_array, 
+		DVDFreeFileSetArrays(sector, title, title_sets_array,
 					sector_sets_array);
 		DVDFreeFileSet(title_set_info);
 		return (0);
 	}
-	
+
 	title_set_info->title_set[0].realsize_bup = fileinfo.st_size;
 	title_set_info->title_set[0].size_bup = bup * 2048;
 	title_set_info->title_set[0].pad_bup = bup - DVDFileSize(vmg_ifo_file);
 
 	/* Take care of the titles which we don't have in VMG */
 
-	title_set_info->title_set[0].number_of_vob_files = 0;	
+	title_set_info->title_set[0].number_of_vob_files = 0;
 	title_set_info->title_set[0].realsize_vob[0] = 0;
 	title_set_info->title_set[0].pad_title = 0;
-	
-	
+
 	DVDCloseFile(vmg_ifo_file);
 
 	if (title_sets >= 1) {
- 		for (counter=0; counter < title_sets; counter++) {
+		for (counter = 0; counter < title_sets; counter++) {
 
 			vts_ifo = ifoOpen(_dvd, counter + 1);
 
-		 	if (!vts_ifo) {
+			if (!vts_ifo) {
 #ifdef	USE_LIBSCHILY
 				errmsgno(EX_BAD, "Can't open VTS info.\n");
 #else
 				fprintf(stderr, "Can't open VTS info.\n");
 #endif
-				DVDFreeFileSetArrays(sector, title, 
+				DVDFreeFileSetArrays(sector, title,
 					title_sets_array, sector_sets_array);
 				DVDFreeFileSet(title_set_info);
 				return (0);
 			}
-		
-			snprintf(temppoint, sizeof(temppoint),
-				"%s/VIDEO_TS/VTS_%02i_0.IFO", 
+
+			snprintf(temppoint, sizeof (temppoint),
+				"%s/VIDEO_TS/VTS_%02i_0.IFO",
 				mountpoint, counter + 1);
 
 			if (stat(temppoint, &fileinfo) < 0) {
 #ifdef	USE_LIBSCHILY
 				errmsg("calc: Can't stat %s\n", temppoint);
 #else
-				fprintf(stderr, "calc: Can't stat %s\n", 
+				fprintf(stderr, "calc: Can't stat %s\n",
 					temppoint);
 				perror("");
 #endif
-				DVDFreeFileSetArrays(sector, title, 
+				DVDFreeFileSetArrays(sector, title,
 					title_sets_array, sector_sets_array);
 				DVDFreeFileSet(title_set_info);
 				return (0);
@@ -427,55 +426,54 @@ DVDGetFileSet(dvd)
 
 			/* Test if VTS_XX_0.VOB is present */
 
-			vts_menu_file = DVDOpenFile(_dvd, counter + 1, 
+			vts_menu_file = DVDOpenFile(_dvd, counter + 1,
 					DVD_READ_MENU_VOBS);
 
 			/* Test if VTS_XX_X.VOB are present */
 
-			vts_title_file = DVDOpenFile(_dvd, counter + 1, 
-					 DVD_READ_TITLE_VOBS);
+			vts_title_file = DVDOpenFile(_dvd, counter + 1,
+						DVD_READ_TITLE_VOBS);
 
 			/* Check VIDEO_TS.IFO */
 
-			vts_ifo_file = DVDOpenFile(_dvd, counter + 1, 
+			vts_ifo_file = DVDOpenFile(_dvd, counter + 1,
 							DVD_READ_INFO_FILE);
-		
 
 			/*
-			 * Checking that title will fit in the 
-			 * space given by the ifo file 
+			 * Checking that title will fit in the
+			 * space given by the ifo file
 			 */
 
 
-			if (vts_ifo->vtsi_mat->vts_last_sector + 1 
-			     < 2 * DVDFileSize(vts_ifo_file)) {
+			if (vts_ifo->vtsi_mat->vts_last_sector + 1
+				< 2 * DVDFileSize(vts_ifo_file)) {
 #ifdef	USE_LIBSCHILY
 				errmsgno(EX_BAD, "IFO is not of correct size aborting.\n");
 #else
 				fprintf(stderr, "IFO is not of correct size aborting\n");
 #endif
-				DVDFreeFileSetArrays(sector, title, 
+				DVDFreeFileSetArrays(sector, title,
 					title_sets_array, sector_sets_array);
 				DVDFreeFileSet(title_set_info);
 				return (0);
-			} else if ((vts_title_file != 0) &&  
-				   (vts_menu_file != 0) && 
-				   (vts_ifo->vtsi_mat->vts_last_sector + 1  
+			} else if ((vts_title_file != 0) &&
+				    (vts_menu_file != 0) &&
+				    (vts_ifo->vtsi_mat->vts_last_sector + 1
 				    < 2 * DVDFileSize(vts_ifo_file) +
-				    DVDFileSize(vts_title_file) + 
+				    DVDFileSize(vts_title_file) +
 				    DVDFileSize(vts_menu_file))) {
 #ifdef	USE_LIBSCHILY
 				errmsgno(EX_BAD, "Either VIDEO_TS.IFO or VIDEO_TS.VOB is not of correct size.\n");
 #else
 				fprintf(stderr, "Either VIDEO_TS.IFO or VIDEO_TS.VOB is not of correct size");
 #endif
-				DVDFreeFileSetArrays(sector, title, 
+				DVDFreeFileSetArrays(sector, title,
 					title_sets_array, sector_sets_array);
 				DVDFreeFileSet(title_set_info);
 				return (0);
-			} else if ((vts_title_file != 0) && 
-				   (vts_menu_file == 0) && 
-				   (vts_ifo->vtsi_mat->vts_last_sector + 1  
+			} else if ((vts_title_file != 0) &&
+				    (vts_menu_file == 0) &&
+				    (vts_ifo->vtsi_mat->vts_last_sector + 1
 				    < 2 * DVDFileSize(vts_ifo_file) +
 				    DVDFileSize(vts_title_file))) {
 #ifdef	USE_LIBSCHILY
@@ -483,13 +481,13 @@ DVDGetFileSet(dvd)
 #else
 				fprintf(stderr, "Either VIDEO_TS.IFO or VIDEO_TS.VOB is not of correct size");
 #endif
-				DVDFreeFileSetArrays(sector, title, 
+				DVDFreeFileSetArrays(sector, title,
 					title_sets_array, sector_sets_array);
 				DVDFreeFileSet(title_set_info);
 				return (0);
-			} else if ((vts_menu_file != 0) && 
-				   (vts_title_file == 0) && 
-				   (vts_ifo->vtsi_mat->vts_last_sector + 1
+			} else if ((vts_menu_file != 0) &&
+				    (vts_title_file == 0) &&
+				    (vts_ifo->vtsi_mat->vts_last_sector + 1
 				    < 2 * DVDFileSize(vts_ifo_file) +
 				    DVDFileSize(vts_menu_file))) {
 #ifdef	USE_LIBSCHILY
@@ -497,43 +495,43 @@ DVDGetFileSet(dvd)
 #else
 				fprintf(stderr, "Either VIDEO_TS.IFO or VIDEO_TS.VOB is not of correct size");
 #endif
-				DVDFreeFileSetArrays(sector, title, 
+				DVDFreeFileSetArrays(sector, title,
 					title_sets_array, sector_sets_array);
 				DVDFreeFileSet(title_set_info);
 				return (0);
 			}
-			
+
 
 			/* Find the actuall right size of VTS_XX_0.IFO */
 			if ((vts_title_file == 0) && (vts_menu_file == 0)) {
-				if (vts_ifo->vtsi_mat->vts_last_sector + 1 > 
+				if (vts_ifo->vtsi_mat->vts_last_sector + 1 >
 				    2 * DVDFileSize(vts_ifo_file)) {
-					ifo=vts_ifo->vtsi_mat->vts_last_sector 
-					    - DVDFileSize(vts_ifo_file) + 1;
+					ifo = vts_ifo->vtsi_mat->vts_last_sector
+						- DVDFileSize(vts_ifo_file) + 1;
 				} else {
-					ifo=vts_ifo->vtsi_mat->vts_last_sector 
-					    - DVDFileSize(vts_ifo_file) + 1;
+					ifo = vts_ifo->vtsi_mat->vts_last_sector
+						- DVDFileSize(vts_ifo_file) + 1;
 				}
 			} else if (vts_title_file == 0) {
-				if (vts_ifo->vtsi_mat->vtsi_last_sector + 1 < 
+				if (vts_ifo->vtsi_mat->vtsi_last_sector + 1 <
 				    vts_ifo->vtsi_mat->vtstt_vobs) {
-					ifo=vmg_ifo->vtsi_mat->vtstt_vobs;
+					ifo = vmg_ifo->vtsi_mat->vtstt_vobs;
 				} else {
-					ifo=vmg_ifo->vtsi_mat->vtstt_vobs;
+					ifo = vmg_ifo->vtsi_mat->vtstt_vobs;
 				}
 			} else {
-				if (vts_ifo->vtsi_mat->vtsi_last_sector + 1 < 
+				if (vts_ifo->vtsi_mat->vtsi_last_sector + 1 <
 				    vts_ifo->vtsi_mat->vtsm_vobs) {
-					ifo=vts_ifo->vtsi_mat->vtsm_vobs;
+					ifo = vts_ifo->vtsi_mat->vtsm_vobs;
 				} else {
-					ifo=vts_ifo->vtsi_mat->vtsi_last_sector + 1;
+					ifo = vts_ifo->vtsi_mat->vtsi_last_sector + 1;
 				}
 			}
-			title_set_info->title_set[counter + 1].size_ifo = 
+			title_set_info->title_set[counter + 1].size_ifo =
 						ifo * 2048;
-			title_set_info->title_set[counter + 1].realsize_ifo = 
+			title_set_info->title_set[counter + 1].realsize_ifo =
 						fileinfo.st_size;
-			title_set_info->title_set[counter + 1].pad_ifo = 
+			title_set_info->title_set[counter + 1].pad_ifo =
 						ifo - DVDFileSize(vts_ifo_file);
 
 
@@ -561,38 +559,38 @@ DVDGetFileSet(dvd)
 #endif
 						return (0);
 					}
-				} else if ((vts_title_file != 0) && 
-				    (vts_ifo->vtsi_mat->vtstt_vobs - 
-				     vts_ifo->vtsi_mat->vtsm_vobs > 
-				     DVDFileSize(vts_menu_file))) {
-					menu_vob=vts_ifo->vtsi_mat->vtstt_vobs -
-						 vts_ifo->vtsi_mat->vtsm_vobs;
-				} else if ((vts_title_file == 0) && 
-					   (vts_ifo->vtsi_mat->vtsm_vobs + 
-					   DVDFileSize(vts_menu_file) + 
-					   DVDFileSize(vts_ifo_file) - 1 < 
-					   vts_ifo->vtsi_mat->vts_last_sector)) {
-					menu_vob=vts_ifo->vtsi_mat->vts_last_sector
-					      - DVDFileSize(vts_ifo_file) 
-					      - vts_ifo->vtsi_mat->vtsm_vobs + 1;
+				} else if ((vts_title_file != 0) &&
+					(vts_ifo->vtsi_mat->vtstt_vobs -
+					vts_ifo->vtsi_mat->vtsm_vobs >
+						DVDFileSize(vts_menu_file))) {
+					menu_vob = vts_ifo->vtsi_mat->vtstt_vobs -
+							vts_ifo->vtsi_mat->vtsm_vobs;
+				} else if ((vts_title_file == 0) &&
+					    (vts_ifo->vtsi_mat->vtsm_vobs +
+					    DVDFileSize(vts_menu_file) +
+					    DVDFileSize(vts_ifo_file) - 1 <
+					    vts_ifo->vtsi_mat->vts_last_sector)) {
+					menu_vob = vts_ifo->vtsi_mat->vts_last_sector
+						- DVDFileSize(vts_ifo_file)
+						- vts_ifo->vtsi_mat->vtsm_vobs + 1;
 				} else {
-					menu_vob=vts_ifo->vtsi_mat->vtstt_vobs -
-						 vts_ifo->vtsi_mat->vtsm_vobs;
+					menu_vob = vts_ifo->vtsi_mat->vtstt_vobs -
+							vts_ifo->vtsi_mat->vtsm_vobs;
 				}
 
-				snprintf(temppoint, sizeof(temppoint),
+				snprintf(temppoint, sizeof (temppoint),
 					"%s/VIDEO_TS/VTS_%02i_0.VOB", mountpoint, counter + 1);
 
 				if (stat(temppoint, &fileinfo)  < 0) {
 #ifdef	USE_LIBSCHILY
 					errmsg("calc: Can't stat %s\n", temppoint);
 #else
-					fprintf(stderr, "calc: Can't stat %s\n", 
+					fprintf(stderr, "calc: Can't stat %s\n",
 						temppoint);
 					perror("");
 #endif
-					DVDFreeFileSetArrays(sector, title, 
-					 title_sets_array, sector_sets_array);
+					DVDFreeFileSetArrays(sector, title,
+						title_sets_array, sector_sets_array);
 					DVDFreeFileSet(title_set_info);
 					return (0);
 				}
@@ -605,7 +603,7 @@ DVDGetFileSet(dvd)
 				title_set_info->title_set[counter + 1].size_menu = 0;
 				title_set_info->title_set[counter + 1].realsize_menu = 0;
 				title_set_info->title_set[counter + 1].pad_menu = 0;
-				menu_vob=0;
+				menu_vob = 0;
 			}
 
 
@@ -613,21 +611,22 @@ DVDGetFileSet(dvd)
 
 			if (vts_title_file != 0) {
 				if (ifo + menu_vob + DVDFileSize(vts_ifo_file) -
- 				    1  < vts_ifo->vtsi_mat->vts_last_sector) {
-				     title_vob=vts_ifo->vtsi_mat->vts_last_sector
-						+ 1 - ifo - menu_vob - 
+				    1 < vts_ifo->vtsi_mat->vts_last_sector) {
+				    title_vob = vts_ifo->vtsi_mat->vts_last_sector
+						+ 1 - ifo - menu_vob -
 						DVDFileSize(vts_ifo_file);
 				} else {
-					title_vob=vts_ifo->vtsi_mat->vts_last_sector +
-					     1 - ifo - menu_vob - 
-					     DVDFileSize(vts_ifo_file);
+					title_vob = vts_ifo->vtsi_mat->vts_last_sector +
+						1 - ifo - menu_vob -
+						DVDFileSize(vts_ifo_file);
 				}
-				/* Find out how many vob files 
-				 * and the size of them 
+				/*
+				 * Find out how many vob files
+				 * and the size of them
 				 */
-				for(i = 0; i < 9; ++i) {
-					snprintf(temppoint, sizeof(temppoint),
-						"%s/VIDEO_TS/VTS_%02i_%i.VOB", 
+				for (i = 0; i < 9; ++i) {
+					snprintf(temppoint, sizeof (temppoint),
+						"%s/VIDEO_TS/VTS_%02i_%i.VOB",
 						mountpoint, counter + 1, i + 1);
 					if (stat(temppoint, &fileinfo) < 0) {
 						break;
@@ -642,45 +641,45 @@ DVDGetFileSet(dvd)
 				title_set_info->title_set[counter + 1].realsize_vob[0] = 0;
 				title_set_info->title_set[counter + 1].size_title = 0;
 				title_set_info->title_set[counter + 1].pad_title = 0;
-				title_vob=0;
+				title_vob = 0;
 
 			}
 
 
 			/* Find the actuall total size of VTS_XX_0.BUP */
 			if (title_sets - 1 > counter) {
-				bup=sector_sets_array[counter+1]
+				bup = sector_sets_array[counter+1]
 					- sector_sets_array[counter]
 					- title_vob - menu_vob - ifo;
 			} else {
-				bup = vts_ifo->vtsi_mat->vts_last_sector + 1 
+				bup = vts_ifo->vtsi_mat->vts_last_sector + 1
 					- title_vob - menu_vob - ifo;
 			}
 
 			/* Never trust the BUP use a copy of the IFO */
-			snprintf(temppoint, sizeof(temppoint),
-				"%s/VIDEO_TS/VTS_%02i_0.IFO", 
+			snprintf(temppoint, sizeof (temppoint),
+				"%s/VIDEO_TS/VTS_%02i_0.IFO",
 				mountpoint, counter + 1);
 
 			if (stat(temppoint, &fileinfo) < 0) {
 #ifdef	USE_LIBSCHILY
 				errmsg("calc: Can't stat %s\n", temppoint);
 #else
-				fprintf(stderr, "calc: Can't stat %s\n", 
+				fprintf(stderr, "calc: Can't stat %s\n",
 					temppoint);
 				perror("");
 #endif
-				DVDFreeFileSetArrays(sector, title, 
+				DVDFreeFileSetArrays(sector, title,
 					title_sets_array, sector_sets_array);
 				DVDFreeFileSet(title_set_info);
 				return (0);
 			}
 
-			title_set_info->title_set[counter + 1].size_bup = 
+			title_set_info->title_set[counter + 1].size_bup =
 						bup * 2048;
-			title_set_info->title_set[counter + 1].realsize_bup = 
+			title_set_info->title_set[counter + 1].realsize_bup =
 						fileinfo.st_size;
-			title_set_info->title_set[counter + 1].pad_bup = 
+			title_set_info->title_set[counter + 1].pad_bup =
 						bup - DVDFileSize(vts_ifo_file);
 
 
@@ -713,7 +712,7 @@ DVDGetFileSet(dvd)
 
 	/* Close the DVD */
 	DVDClose(_dvd);
-	
+
 	/* Return the actuall info*/
 	return (title_set_info);
 
@@ -751,7 +750,7 @@ DVDGetFilePad(title_set_info, name)
 		title_a[1] = name[5];
 		title_a[2] = '\0';
 		vob_a[0] = name[7];
-		vob_a[1] = '\0'; 
+		vob_a[1] = '\0';
 		title = atoi(title_a);
 		vob = atoi(vob_a);
 		if (title > title_set_info->num_titles) {
