@@ -1,31 +1,32 @@
-/* @(#)eltorito.c	1.24 02/05/30 joerg */
+/* @(#)eltorito.c	1.32 04/05/23 joerg */
 #ifndef lint
 static	char sccsid[] =
-	"@(#)eltorito.c	1.24 02/05/30 joerg";
+	"@(#)eltorito.c	1.32 04/05/23 joerg";
 
 #endif
 /*
  * Program eltorito.c - Handle El Torito specific extensions to iso9660.
  *
-
-   Written by Michael Fulbright <msf@redhat.com> (1996).
-
-   Copyright 1996 RedHat Software, Incorporated
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.   */
-
+ *
+ *  Written by Michael Fulbright <msf@redhat.com> (1996).
+ *
+ * Copyright 1996 RedHat Software, Incorporated
+ * Copyright (c) 1999-2004 J. Schilling
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
 
 #include <mconfig.h>
 #include "mkisofs.h"
@@ -38,7 +39,7 @@ static	char sccsid[] =
 #include <schily.h>
 
 #undef MIN
-#define MIN(a, b) (((a) < (b))? (a): (b))
+#define	MIN(a, b) (((a) < (b))? (a): (b))
 
 static struct eltorito_validation_entry valid_desc;
 static struct eltorito_boot_descriptor gboot_desc;
@@ -63,6 +64,24 @@ void
 init_boot_catalog(path)
 	const char	*path;
 {
+#ifdef	SORTING
+	struct eltorito_boot_entry_info * cbe;
+
+	for (cbe = first_boot_entry;
+	    cbe != NULL;
+	    cbe = cbe->next) {
+		char	*p;
+
+		p = (char *) e_malloc(strlen(cbe->boot_image) + strlen(path) + 2);
+		strcpy(p, path);
+		if (p[strlen(p) - 1] != '/') {
+			strcat(p, "/");
+		}
+		strcat(p, cbe->boot_image);
+		add_sort_match(p, sort_matches(p, 1));
+		free(p);
+	}
+#endif
 	bootcat_path = (char *) e_malloc(strlen(boot_catalog) + strlen(path) + 2);
 	strcpy(bootcat_path, path);
 	if (bootcat_path[strlen(bootcat_path) - 1] != '/') {
@@ -96,14 +115,14 @@ init_boot_catalog(path)
 void
 insert_boot_cat()
 {
-	struct directory_entry *de,
-	               *s_entry;
-	char           *p1,
-	               *p2,
-	               *p3;
-	struct directory *this_dir,
-	               *dir;
-	char           *buffer;
+	struct directory_entry	*de;
+	struct directory_entry	*s_entry;
+	char			*p1;
+	char			*p2;
+	char			*p3;
+	struct directory	*this_dir;
+	struct directory	*dir;
+	char			*buffer;
 
 	init_fstatbuf();
 
@@ -171,14 +190,15 @@ insert_boot_cat()
 	 * entries
 	 */
 	s_entry = (struct directory_entry *)
-		e_malloc(sizeof(struct directory_entry));
-	memset(s_entry, 0, sizeof(struct directory_entry));
+		e_malloc(sizeof (struct directory_entry));
+	memset(s_entry, 0, sizeof (struct directory_entry));
 	s_entry->next = this_dir->contents;
 	this_dir->contents = s_entry;
 
 #ifdef SORTING
 	/* inherit any sort weight from parent directory */
 	s_entry->sort = this_dir->sort;
+	s_entry->sort += 2;
 
 	/* see if this entry should have a new weighting */
 	if (do_sort) {
@@ -202,10 +222,11 @@ insert_boot_cat()
 	/* flag file as necessary */
 	s_entry->de_flags = bcat_de_flags;
 
-	if (use_RockRidge && !(bcat_de_flags & INHIBIT_ISO9660_ENTRY)) {
+	if ((use_XA || use_RockRidge) &&
+	    !(bcat_de_flags & INHIBIT_ISO9660_ENTRY)) {
 		fstatbuf.st_mode = 0444 | S_IFREG;
 		fstatbuf.st_nlink = 1;
-		generate_rock_ridge_attributes("",
+		generate_xa_rr_attributes("",
 			p2, s_entry,
 			&fstatbuf, &fstatbuf, 0);
 	}
@@ -225,7 +246,7 @@ insert_boot_cat()
 		/*
 		 * store the (empty) file data in the
 		 * unused s_entry->whole_name element for the time being
-	 	 * - this will be transferred to s_entry->table after any
+		 * - this will be transferred to s_entry->table after any
 		 * TRANS.TBL processing later
 		 */
 		s_entry->whole_name = buffer;
@@ -240,19 +261,19 @@ static void
 get_torito_desc(boot_desc)
 	struct eltorito_boot_descriptor	*boot_desc;
 {
-	int             checksum;
-	unsigned char  *checksum_ptr;
-	struct directory_entry *de2;	/* Boot catalog */
-	int		i;
-	int		offset;
+	int			checksum;
+	unsigned char		*checksum_ptr;
+	struct directory_entry	*de2;	/* Boot catalog */
+	int			i;
+	int			offset;
 	struct eltorito_defaultboot_entry boot_desc_record;
 
-	memset(boot_desc, 0, sizeof(*boot_desc));
+	memset(boot_desc, 0, sizeof (*boot_desc));
 	boot_desc->type[0] = 0;
-	memcpy(boot_desc->id, ISO_STANDARD_ID, sizeof(ISO_STANDARD_ID));
+	memcpy(boot_desc->id, ISO_STANDARD_ID, sizeof (ISO_STANDARD_ID));
 	boot_desc->version[0] = 1;
 
-	memcpy(boot_desc->system_id, EL_TORITO_ID, sizeof(EL_TORITO_ID));
+	memcpy(boot_desc->system_id, EL_TORITO_ID, sizeof (EL_TORITO_ID));
 
 	/*
 	 * search from root of iso fs to find boot catalog
@@ -278,7 +299,7 @@ get_torito_desc(boot_desc)
 	 * we have the boot image, so write boot catalog information
 	 * Next we write out the primary descriptor for the disc
 	 */
-	memset(&valid_desc, 0, sizeof(valid_desc));
+	memset(&valid_desc, 0, sizeof (valid_desc));
 	valid_desc.headerid[0] = 1;
 	valid_desc.arch[0] = EL_TORITO_ARCH_x86;
 
@@ -298,7 +319,7 @@ get_torito_desc(boot_desc)
 	checksum_ptr = (unsigned char *) &valid_desc;
 	/* Set checksum to 0 before computing checksum */
 	set_721(valid_desc.cksum, 0);
-	for (i = 0; i < (int)sizeof(valid_desc); i += 2) {
+	for (i = 0; i < (int)sizeof (valid_desc); i += 2) {
 		checksum += (unsigned int) checksum_ptr[i];
 		checksum += ((unsigned int) checksum_ptr[i + 1]) * 256;
 	}
@@ -310,10 +331,10 @@ get_torito_desc(boot_desc)
 	/* now write it to the virtual boot catalog */
 	memcpy(de2->table, &valid_desc, 32);
 
-	for (current_boot_entry = first_boot_entry,offset = sizeof(valid_desc);
-	     current_boot_entry != NULL;
-	     current_boot_entry = current_boot_entry->next,
-	     offset += sizeof(boot_desc_record)) {
+	for (current_boot_entry = first_boot_entry, offset = sizeof (valid_desc);
+		current_boot_entry != NULL;
+		current_boot_entry = current_boot_entry->next,
+		offset += sizeof (boot_desc_record)) {
 
 		if (offset >= SECTOR_SIZE) {
 #ifdef	USE_LIBSCHILY
@@ -325,9 +346,9 @@ get_torito_desc(boot_desc)
 			exit(1);
 #endif
 		}
-		fill_boot_desc (&boot_desc_record, current_boot_entry);
+		fill_boot_desc(&boot_desc_record, current_boot_entry);
 		memcpy(de2->table + offset, &boot_desc_record,
-					sizeof(boot_desc_record));
+					sizeof (boot_desc_record));
 	}
 }/* get_torito_desc(... */
 
@@ -336,11 +357,11 @@ fill_boot_desc(boot_desc_entry, boot_entry)
 	struct eltorito_defaultboot_entry *boot_desc_entry;
 	struct eltorito_boot_entry_info *boot_entry;
 {
-	struct directory_entry *de;	/* Boot file */
-	int             bootmbr;
-	int             i;
-	int             nsectors;
-	int             geosec;
+	struct directory_entry	*de;	/* Boot file */
+	int			bootmbr;
+	int			i;
+	int			nsectors;
+	int			geosec;
 
 	if (!boot_desc_entry || !boot_entry)
 		return;
@@ -358,7 +379,7 @@ fill_boot_desc(boot_desc_entry, boot_entry)
 #endif
 	}
 	/* now make the initial/default entry for boot catalog */
-	memset(boot_desc_entry, 0, sizeof(*boot_desc_entry));
+	memset(boot_desc_entry, 0, sizeof (*boot_desc_entry));
 	boot_desc_entry->boot_id[0] = (char) boot_entry->not_bootable ?
 				EL_TORITO_NOT_BOOTABLE : EL_TORITO_BOOTABLE;
 
@@ -373,12 +394,16 @@ fill_boot_desc(boot_desc_entry, boot_entry)
 	nsectors = boot_entry->load_size ? boot_entry->load_size :
 				ISO_BLOCKS(de->size) * (SECTOR_SIZE/512);
 
-	fprintf(stderr, "\nSize of boot image is %d sectors -> ", nsectors);
+	if (verbose > 0) {
+		fprintf(stderr,
+			"Size of boot image is %d sectors -> ", nsectors);
+	}
 
 	if (boot_entry->hard_disk_boot) {
 		/* sanity test hard disk boot image */
 		boot_desc_entry->boot_media[0] = EL_TORITO_MEDIA_HD;
-		fprintf(stderr, "Emulating a hard disk\n");
+		if (verbose > 0)
+			fprintf(stderr, "Emulating a hard disk\n");
 
 		/* read MBR */
 		bootmbr = open(de->whole_name, O_RDONLY | O_BINARY);
@@ -394,8 +419,8 @@ fill_boot_desc(boot_desc_entry, boot_entry)
 			exit(1);
 #endif
 		}
-		if (read(bootmbr, &disk_mbr, sizeof(disk_mbr)) !=
-							sizeof(disk_mbr)) {
+		if (read(bootmbr, &disk_mbr, sizeof (disk_mbr)) !=
+							sizeof (disk_mbr)) {
 #ifdef	USE_LIBSCHILY
 			comerr("Error reading MBR from boot image '%s'.\n",
 							de->whole_name);
@@ -421,8 +446,8 @@ fill_boot_desc(boot_desc_entry, boot_entry)
 		/* find partition type */
 		boot_desc_entry->sys_type[0] = PARTITION_UNUSED;
 		for (i = 0; i < PARTITION_COUNT; ++i) {
-			int             s_cyl_sec;
-			int             e_cyl_sec;
+			int		s_cyl_sec;
+			int		e_cyl_sec;
 
 			s_cyl_sec =
 			la_to_u_2_byte(disk_mbr.partition[i].s_cyl_sec);
@@ -487,7 +512,7 @@ fill_boot_desc(boot_desc_entry, boot_entry)
 #else
 			fprintf(stderr,
 					"Boot image '%s' has no partitions.\n",
-							 de->whole_name);
+							de->whole_name);
 			exit(1);
 #endif
 		}
@@ -504,30 +529,34 @@ fill_boot_desc(boot_desc_entry, boot_entry)
 		 * in the boot image
 		 */
 		boot_desc_entry->boot_media[0] = EL_TORITO_MEDIA_NOEMUL;
-		fprintf(stderr, "No emulation\n");
+		if (verbose > 0)
+			fprintf(stderr, "No emulation\n");
 
 	} else {
 		/* choose size of emulated floppy based on boot image size */
 		if (nsectors == 2880) {
 			boot_desc_entry->boot_media[0] = EL_TORITO_MEDIA_144FLOP;
-			fprintf(stderr, "Emulating a 1440 kB floppy\n");
+			if (verbose > 0)
+				fprintf(stderr, "Emulating a 1440 kB floppy\n");
 
 		} else if (nsectors == 5760) {
 			boot_desc_entry->boot_media[0] = EL_TORITO_MEDIA_288FLOP;
-			fprintf(stderr, "Emulating a 2880 kB floppy\n");
+			if (verbose > 0)
+				fprintf(stderr, "Emulating a 2880 kB floppy\n");
 
 		} else if (nsectors == 2400) {
 			boot_desc_entry->boot_media[0] = EL_TORITO_MEDIA_12FLOP;
-			fprintf(stderr, "Emulating a 1200 kB floppy\n");
+			if (verbose > 0)
+				fprintf(stderr, "Emulating a 1200 kB floppy\n");
 
 		} else {
 #ifdef	USE_LIBSCHILY
 			comerrno(EX_BAD,
-			"Error - boot image '%s' is not the an allowable size.\n",
+			"Error - boot image '%s' has not an allowable size.\n",
 							de->whole_name);
 #else
 			fprintf(stderr,
-			"Error - boot image '%s' is not the an allowable size.\n",
+			"Error - boot image '%s' has not an allowable size.\n",
 							de->whole_name);
 			exit(1);
 #endif
@@ -550,11 +579,11 @@ fill_boot_desc(boot_desc_entry, boot_entry)
 
 	/* If the user has asked for it, patch the boot image */
 	if (boot_entry->boot_info_table) {
-		int             bootimage;
-		unsigned int    bi_checksum,
-		                total_len;
-		static char     csum_buffer[SECTOR_SIZE];
-		int             len;
+		int		bootimage;
+		unsigned int	bi_checksum;
+		unsigned int	total_len;
+		static char	csum_buffer[SECTOR_SIZE];
+		int		len;
 		struct mkisofs_boot_info bi_table;
 
 		bootimage = open(de->whole_name, O_RDWR | O_BINARY);
@@ -610,14 +639,14 @@ fill_boot_desc(boot_desc_entry, boot_entry)
 		}
 		/* End of file, set position to byte 8 */
 		lseek(bootimage, (off_t)8, SEEK_SET);
-		memset(&bi_table, 0, sizeof(bi_table));
+		memset(&bi_table, 0, sizeof (bi_table));
 		/* Is it always safe to assume PVD is at session_start+16? */
 		set_731(bi_table.bi_pvd, session_start + 16);
 		set_731(bi_table.bi_file, de->starting_block);
 		set_731(bi_table.bi_length, de->size);
 		set_731(bi_table.bi_csum, bi_checksum);
 
-		write(bootimage, &bi_table, sizeof(bi_table));
+		write(bootimage, &bi_table, sizeof (bi_table));
 		close(bootimage);
 	}
 }/* fill_boot_desc(... */
@@ -628,9 +657,9 @@ get_boot_entry()
 	if (current_boot_entry)
 		return;
 
-	current_boot_entry = (struct eltorito_boot_entry_info*)
-			e_malloc(sizeof(struct eltorito_boot_entry_info));
-	memset(current_boot_entry, 0, sizeof(*current_boot_entry));
+	current_boot_entry = (struct eltorito_boot_entry_info *)
+			e_malloc(sizeof (struct eltorito_boot_entry_info));
+	memset(current_boot_entry, 0, sizeof (*current_boot_entry));
 
 	if (!first_boot_entry) {
 		first_boot_entry = current_boot_entry;
@@ -665,9 +694,9 @@ tvd_write(outfile)
 	}
 	/* Next we write out the boot volume descriptor for the disc */
 	get_torito_desc(&gboot_desc);
-	xfwrite(&gboot_desc, 1, SECTOR_SIZE, outfile);
+	xfwrite(&gboot_desc, SECTOR_SIZE, 1, outfile, 0, FALSE);
 	last_extent_written++;
-	return 0;
+	return (0);
 }
 
 struct output_fragment torito_desc = {NULL, oneblock_size, NULL, tvd_write, "Eltorito Volume Descriptor"};
