@@ -1,12 +1,12 @@
-/* @(#)misc.c	1.2 01/10/29 Copyright 1998 J. Schilling */
+/* @(#)misc.c	1.4 04/03/02 Copyright 1998, 2001-2004 J. Schilling */
 #ifndef lint
 static	char sccsid[] =
-	"@(#)misc.c	1.2 01/10/29 Copyright 1998 J. Schilling";
+	"@(#)misc.c	1.4 04/03/02 Copyright 1998, 2001-2004 J. Schilling";
 #endif
 /*
  *	Misc support functions
  *
- *	Copyright (c) 1998 J. Schilling
+ *	Copyright (c) 1998, 2001-2004 J. Schilling
  */
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -19,16 +19,21 @@ static	char sccsid[] =
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License along with
+ * this program; see the file COPYING.  If not, write to the Free Software
+ * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #include <mconfig.h>
 #include <timedefs.h>
+#include <stdio.h>
 #include <standard.h>
+#include <schily.h>
 
 EXPORT	void	timevaldiff	__PR((struct timeval *start, struct timeval *stop));
+EXPORT	void	prtimediff	__PR((const char *fmt,
+					struct timeval *start,
+					struct timeval *stop));
 
 EXPORT void
 timevaldiff(start, stop)
@@ -49,3 +54,29 @@ timevaldiff(start, stop)
 	}
 	*stop = tv;
 }
+
+EXPORT void
+prtimediff(fmt, start, stop)
+	const	char	*fmt;
+	struct timeval	*start;
+	struct timeval	*stop;
+{
+	struct timeval tv;
+
+	tv.tv_sec = stop->tv_sec - start->tv_sec;
+	tv.tv_usec = stop->tv_usec - start->tv_usec;
+	while (tv.tv_usec > 1000000) {
+		tv.tv_usec -= 1000000;
+		tv.tv_sec += 1;
+	}
+	while (tv.tv_usec < 0) {
+		tv.tv_usec += 1000000;
+		tv.tv_sec -= 1;
+	}
+	/*
+	 * We need to cast timeval->* to long because
+	 * of the broken sys/time.h in Linux.
+	 */
+	printf("%s%4ld.%03lds\n", fmt, (long)tv.tv_sec, (long)tv.tv_usec/1000);
+	flush();
+}
