@@ -1,12 +1,12 @@
-/* @(#)readcd.c	1.53 02/11/30 Copyright 1987, 1995-2002 J. Schilling */
+/* @(#)readcd.c	1.74 04/09/08 Copyright 1987, 1995-2004 J. Schilling */
 #ifndef lint
 static	char sccsid[] =
-	"@(#)readcd.c	1.53 02/11/30 Copyright 1987, 1995-2002 J. Schilling";
+	"@(#)readcd.c	1.74 04/09/08 Copyright 1987, 1995-2004 J. Schilling";
 #endif
 /*
  *	Skeleton for the use of the scg genearal SCSI - driver
  *
- *	Copyright (c) 1987, 1995-2002 J. Schilling
+ *	Copyright (c) 1987, 1995-2004 J. Schilling
  */
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -35,28 +35,34 @@ static	char sccsid[] =
 #include <signal.h>
 #include <schily.h>
 
+#ifdef	NEED_O_BINARY
+#include <io.h>					/* for setmode() prototype */
+#endif
+
 #include <scg/scgcmd.h>
 #include <scg/scsireg.h>
 #include <scg/scsitransp.h>
 
+#include "scsi_scan.h"
 #include "scsimmc.h"
 #define	qpto96	__nothing__
 #include "cdrecord.h"
+#include "defaults.h"
 #undef	qpto96
 #include "movesect.h"
 
-char	cdr_version[] = "2.00.3";
+char	cdr_version[] = "2.01";
 
-#if     defined(PROTOTYPES)
-#define UINT_C(a)	(a##u)
-#define ULONG_C(a)	(a##ul)
-#define USHORT_C(a)	(a##uh)
-#define CONCAT(a,b)	a##b
+#if	defined(PROTOTYPES)
+#define	UINT_C(a)	(a##u)
+#define	ULONG_C(a)	(a##ul)
+#define	USHORT_C(a)	(a##uh)
+#define	CONCAT(a, b)	a##b
 #else
-#define UINT_C(a)	((unsigned)(a))
-#define ULONG_C(a)	((unsigned long)(a))
-#define USHORT_C(a)	((unsigned short)(a))
-#define CONCAT(a,b)	a/**/b
+#define	UINT_C(a)	((unsigned)(a))
+#define	ULONG_C(a)	((unsigned long)(a))
+#define	USHORT_C(a)	((unsigned short)(a))
+#define	CONCAT(a, b)	a/**/b
 #endif
 
 extern	BOOL	getlong		__PR((char *, long *, long, long));
@@ -97,12 +103,27 @@ LOCAL	void	intr		__PR((int sig));
 LOCAL	void	exscsi		__PR((int excode, void *arg));
 LOCAL	void	excdr		__PR((int excode, void *arg));
 LOCAL	int	prstats		__PR((void));
-LOCAL	void	dorw		__PR((SCSI *scgp, char *filename, char* sectors));
+LOCAL	int	prstats_silent	__PR((void));
+LOCAL	void	dorw		__PR((SCSI *scgp, char *filename, char *sectors));
 LOCAL	void	doit		__PR((SCSI *scgp));
 LOCAL	void	read_disk	__PR((SCSI *scgp, parm_t *parmp));
+#ifdef	CLONE_WRITE
+LOCAL	void	readcd_disk	__PR((SCSI *scgp, parm_t *parmp));
+LOCAL	void	read_lin	__PR((SCSI *scgp, parm_t *parmp));
+LOCAL	int	read_secheader	__PR((SCSI *scgp, long addr));
+LOCAL	int	read_ftoc	__PR((SCSI *scgp, parm_t *parmp, BOOL do_sectype));
+LOCAL	void	read_sectypes	__PR((SCSI *scgp, FILE *f));
+LOCAL	void	get_sectype	__PR((SCSI *scgp, long addr, char *st));
+#endif
 
 LOCAL	void	readc2_disk	__PR((SCSI *scgp, parm_t *parmp));
 LOCAL	int	fread_data	__PR((SCSI *scgp, rparm_t *rp, caddr_t bp, long addr, int cnt));
+#ifdef	CLONE_WRITE
+LOCAL	int	fread_2448	__PR((SCSI *scgp, rparm_t *rp, caddr_t bp, long addr, int cnt));
+LOCAL	int	fread_2448_16	__PR((SCSI *scgp, rparm_t *rp, caddr_t bp, long addr, int cnt));
+LOCAL	int	fread_2352	__PR((SCSI *scgp, rparm_t *rp, caddr_t bp, long addr, int cnt));
+LOCAL	int	fread_lin	__PR((SCSI *scgp, rparm_t *rp, caddr_t bp, long addr, int cnt));
+#endif
 LOCAL	int	bits		__PR((int c));
 LOCAL	int	bitidx		__PR((int c));
 LOCAL	int	fread_c2	__PR((SCSI *scgp, rparm_t *rp, caddr_t bp, long addr, int cnt));
@@ -123,8 +144,7 @@ LOCAL	void	Xrequest_sense	__PR((SCSI *scgp));
 #endif
 LOCAL	int	read_retry	__PR((SCSI *scgp, caddr_t bp, long addr, long cnt,
 					int (*rfunc)(SCSI *scgp, rparm_t *rp, caddr_t bp, long addr, int cnt),
-					rparm_t *rp
-					));
+					rparm_t *rp));
 LOCAL	void	read_generic	__PR((SCSI *scgp, parm_t *parmp,
 					int (*rfunc)(SCSI *scgp, rparm_t *rp, caddr_t bp, long addr, int cnt),
 					rparm_t *rp,
@@ -142,6 +162,8 @@ LOCAL	void	domode		__PR((SCSI *scgp, int err, int retr));
 
 LOCAL	void	qpto96		__PR((Uchar *sub, Uchar *subq, int dop));
 LOCAL	void	ovtime		__PR((SCSI *scgp));
+LOCAL	void	add_bad		__PR((long addr));
+LOCAL	void	print_bad	__PR((void));
 
 struct timeval	starttime;
 struct timeval	stoptime;
@@ -160,6 +182,7 @@ int	lverbose;
 int	quiet;
 BOOL	is_suid;
 BOOL	is_cdrom;
+BOOL	is_dvd;
 BOOL	do_write;
 BOOL	c2scan;
 BOOL	fulltoc;
@@ -168,12 +191,15 @@ BOOL	noerror;
 BOOL	nocorr;
 BOOL	notrunc;
 int	retries = MAX_RETRY;
+int	maxtry = 0;
+int	meshpoints;
+BOOL	do_factor;
 
 struct	scsi_format_data fmt;
 
-/*XXX*/EXPORT	BOOL cvt_cyls(){ return (FALSE);}
-/*XXX*/EXPORT	BOOL cvt_bcyls(){ return (FALSE);}
-/*XXX*/EXPORT	void print_defect_list(){}
+/*XXX*/EXPORT	BOOL cvt_cyls() { return (FALSE); }
+/*XXX*/EXPORT	BOOL cvt_bcyls() { return (FALSE); }
+/*XXX*/EXPORT	void print_defect_list() {}
 
 LOCAL void
 usage(ret)
@@ -186,8 +212,13 @@ usage(ret)
 	error("\tf=filename	Name of file to read/write\n");
 	error("\tsectors=range	Range of sectors to read/write\n");
 	error("\tspeed=#		set speed of drive (MMC only)\n");
+	error("\tts=#		set maximum transfer size for a single SCSI command\n");
 	error("\t-w		Switch to write mode\n");
 	error("\t-c2scan		Do a C2 error scan\n");
+#ifdef	CLONE_WRITE
+	error("\t-fulltoc	Retrieve the full TOC\n");
+	error("\t-clone		Retrieve the full TOC and all data\n");
+#endif
 	error("\ttimeout=#	set the default SCSI command timeout to #.\n");
 	error("\tdebug=#,-d	Set to # or increment misc debug level\n");
 	error("\tkdebug=#,kd=#	do Kernel debugging\n");
@@ -195,16 +226,22 @@ usage(ret)
 	error("\t-verbose,-v	increment general verbose level by one\n");
 	error("\t-Verbose,-V	increment SCSI command transport verbose level by one\n");
 	error("\t-silent,-s	do not print status of failed SCSI commands\n");
+	error("\t-scanbus	scan the SCSI bus and exit\n");
 	error("\t-noerror	do not abort on error\n");
+#ifdef	CLONE_WRITE
+	error("\t-nocorr		do not apply error correction in drive\n");
+#endif
 	error("\t-notrunc	do not truncate outputfile in read mode\n");
 	error("\tretries=#	set retry count (default is %d)\n", retries);
 	error("\t-overhead	meter SCSI command overhead times\n");
+	error("\tmeshpoints=#	print read-speed at # locations\n");
+	error("\t-factor		try to use speed factor with meshpoints=# if possible\n");
 	error("\n");
 	error("sectors=0-0 will read nothing, sectors=0-1 will read one sector starting from 0\n");
 	exit(ret);
 }	
 
-char	opts[]   = "debug#,d+,kdebug#,kd#,timeout#,quiet,q,verbose+,v+,Verbose+,V+,x+,xd#,silent,s,help,h,version,dev*,sectors*,w,c2scan,fulltoc,clone,noerror,nocorr,notrunc,retries#,f*,speed#,overhead";
+char	opts[]   = "debug#,d+,kdebug#,kd#,timeout#,quiet,q,verbose+,v+,Verbose+,V+,x+,xd#,silent,s,help,h,version,scanbus,dev*,sectors*,w,c2scan,fulltoc,clone,noerror,nocorr,notrunc,retries#,factor,f*,speed#,ts&,overhead,meshpoints#";
 
 EXPORT int
 main(ac, av)
@@ -222,12 +259,13 @@ main(ac, av)
 	int	verbose	= 0;
 	int	kdebug	= 0;
 	int	debug	= 0;
-	int	deftimeout= 40;
+	int	deftimeout = 40;
 	int	pversion = 0;
+	int	scanbus = 0;
 	int	speed	= -1;
-	int	dooverhead=0;
+	int	dooverhead = 0;
 	SCSI	*scgp;
-	char	*filename= NULL;
+	char	*filename = NULL;
 	char	*sectors = NULL;
 
 	save_args(ac, av);
@@ -235,7 +273,7 @@ main(ac, av)
 	cac = --ac;
 	cav = ++av;
 
-	if(getallargs(&cac, &cav, opts,
+	if (getallargs(&cac, &cav, opts,
 			&debug, &debug,
 			&kdebug, &kdebug,
 			&deftimeout,
@@ -245,19 +283,20 @@ main(ac, av)
 			&xdebug, &xdebug,
 			&silent, &silent,
 			&help, &help, &pversion,
-			&dev, &sectors, &do_write,
+			&scanbus, &dev, &sectors, &do_write,
 			&c2scan,
 			&fulltoc, &clone,
 			&noerror, &nocorr,
-			&notrunc, &retries, &filename,
-			&speed, &dooverhead) < 0) {
+			&notrunc, &retries, &do_factor, &filename,
+			&speed, getnum, &Sbufsize,
+			&dooverhead, &meshpoints) < 0) {
 		errmsgno(EX_BAD, "Bad flag: %s.\n", cav[0]);
 		usage(EX_BAD);
 	}
 	if (help)
 		usage(0);
 	if (pversion) {
-		printf("readcd %s (%s-%s-%s) Copyright (C) 1987, 1995-2002 Jrg Schilling\n",
+		printf("readcd %s (%s-%s-%s) Copyright (C) 1987, 1995-2003 Jrg Schilling\n",
 								cdr_version,
 								HOST_CPU, HOST_VENDOR, HOST_OS);
 		exit(0);
@@ -267,7 +306,7 @@ main(ac, av)
 	cac = ac;
 	cav = av;
 
-	while(getfiles(&cac, &cav, opts) > 0) {
+	while (getfiles(&cac, &cav, opts) > 0) {
 		fcount++;
 		if (fcount == 1) {
 			if (*astoi(cav[0], &target) != '\0') {
@@ -303,7 +342,8 @@ main(ac, av)
 	}
 /*error("dev: '%s'\n", dev);*/
 
-	cdr_defaults(&dev, NULL, NULL, NULL);
+	if (!scanbus)
+		cdr_defaults(&dev, NULL, NULL, NULL);
 	if (debug) {
 		printf("dev: '%s'\n", dev);
 	}
@@ -321,10 +361,12 @@ main(ac, av)
 			scg_help(stderr);
 			exit(0);
 		}
-		if ((scgp = scg_open(dev, errstr, sizeof(errstr), debug, lverbose)) == (SCSI *)0) {
+		if ((scgp = scg_open(dev, errstr, sizeof (errstr), debug, lverbose)) == (SCSI *)0) {
 			int	err = geterrno();
 
 			errmsgno(err, "%s%sCannot open SCSI driver.\n", errstr, errstr[0]?". ":"");
+			errmsgno(EX_BAD, "For possible targets try 'readcd -scanbus'.%s\n",
+						geteuid() ? " Make sure you are root.":"");
 			errmsgno(EX_BAD, "For possible transport specifiers try 'readcd dev=help'.\n");
 			exit(err);
 		}
@@ -346,7 +388,9 @@ main(ac, av)
 	scgp->kdebug = kdebug;
 	scg_settimeout(scgp, deftimeout);
 
-	Sbufsize = scg_bufsize(scgp, 256*1024L);
+	if (Sbufsize == 0)
+		Sbufsize = 256*1024L;
+	Sbufsize = scg_bufsize(scgp, Sbufsize);
 	if ((Sbuf = scg_getbuf(scgp, Sbufsize)) == NULL)
 		comerr("Cannot get SCSI I/O buffer.\n");
 
@@ -367,6 +411,10 @@ main(ac, av)
 
 	/* code to use SCG */
 
+	if (scanbus) {
+		select_target(scgp, stdout);
+		exit(0);
+	}
 	do_inquiry(scgp, FALSE);
 	allow_atapi(scgp, TRUE);    /* Try to switch to 10 byte mode cmds */
 	if (is_mmc(scgp, NULL, NULL)) {
@@ -382,6 +430,33 @@ main(ac, av)
 		if (scgp->inq->data_format < 2)
 			scgp->inq->data_format = 2;
 
+		if ((rspeed = get_curprofile(scgp)) >= 0) {
+			if (rspeed >= 0x08 && rspeed < 0x10)
+				is_cdrom = TRUE;
+			if (rspeed >= 0x10 && rspeed < 0x20)
+				is_dvd = TRUE;
+		} else {
+			BOOL	dvd;
+
+			mmc_check(scgp, NULL, NULL, NULL, NULL, &dvd, NULL);
+			if (dvd == FALSE) {
+				is_cdrom = TRUE;
+			} else {
+				char	xb[32];
+
+				if (read_dvd_structure(scgp, (caddr_t)xb, 32, 0, 0, 0) >= 0) {
+				/*
+				 * If read DVD structure is supported and works, then
+				 * we must have a DVD media in the drive. Signal to
+				 * use the DVD driver.
+				 */
+					is_dvd = TRUE;
+				} else {
+					is_cdrom = TRUE;
+				}
+			}
+		}
+
 		if (speed > 0)
 			speed *= 177;
 		if (speed > 0xFFFF || speed < 0)
@@ -397,7 +472,7 @@ main(ac, av)
 	exargs.scgp	   = scgp;
 	exargs.old_secsize = -1;
 /*	exargs.flags	   = flags;*/
-	exargs.oerr[2]     = 0;
+	exargs.oerr[2]	   = 0;
 
 	/*
 	 * Install exit handler before we change the drive status.
@@ -416,7 +491,7 @@ main(ac, av)
 			comerrno(EX_BAD, "Not root. Will only work on CD-ROM in suid mode\n");
 	}
 
-	if (filename || sectors || c2scan || fulltoc || clone) {
+	if (filename || sectors || c2scan || meshpoints || fulltoc || clone) {
 		dorw(scgp, filename, sectors);
 	} else {
 		doit(scgp);
@@ -452,7 +527,7 @@ exscsi(excode, arg)
 	 * Try to restore the old sector size.
 	 */
 	if (exp != NULL && exp->exflags == 0) {
-		for (i=0; i < 10*100; i++) {
+		for (i = 0; i < 10*100; i++) {
 			if (!exp->scgp->running)
 				break;
 			if (i == 10) {
@@ -513,6 +588,33 @@ prstats()
 	return (1000*sec + (usec / 1000));
 }
 
+/*
+ * Return milliseconds since start time, but be silent this time.
+ */
+LOCAL int
+prstats_silent()
+{
+	int	sec;
+	int	usec;
+	int	tmsec;
+
+	if (gettimeofday(&stoptime, (struct timezone *)0) < 0)
+		comerr("Cannot get time\n");
+
+	sec = stoptime.tv_sec - starttime.tv_sec;
+	usec = stoptime.tv_usec - starttime.tv_usec;
+	tmsec = sec*1000 + usec/1000;
+#ifdef	lint
+	tmsec = tmsec;	/* Bisz spaeter */
+#endif
+	if (usec < 0) {
+		sec--;
+		usec += 1000000;
+	}
+
+	return (1000*sec + (usec / 1000));
+}
+
 LOCAL void
 dorw(scgp, filename, sectors)
 	SCSI	*scgp;
@@ -530,6 +632,10 @@ dorw(scgp, filename, sectors)
 
 	if (filename)
 		params.name = filename;
+	if (meshpoints > 0) {
+		if (params.name == NULL)
+			params.name = "/dev/null";
+	}
 	if (sectors)
 		p = astol(sectors, &params.start);
 	if (p && *p == '-')
@@ -540,6 +646,25 @@ dorw(scgp, filename, sectors)
 	if (!wait_unit_ready(scgp, 60))
 		comerrno(EX_BAD, "Device not ready.\n");
 
+#ifdef	CLONE_WRITE
+	if (fulltoc) {
+		if (params.name == NULL)
+			params.name = "/dev/null";
+		read_ftoc(scgp, &params, FALSE);
+	} else if (clone) {
+		if (!is_mmc(scgp, NULL, NULL))
+			comerrno(EX_BAD, "Unsupported device for clone mode.\n");
+		noerror = TRUE;
+		if (retries == MAX_RETRY)
+			retries = 10;
+		if (params.name == NULL)
+			params.name = "/dev/null";
+
+		if (read_ftoc(scgp, &params, TRUE) < 0)
+			comerrno(EX_BAD, "Read fulltoc problems.\n");
+		readcd_disk(scgp, &params);
+	} else
+#endif
 	if (c2scan) {
 		noerror = TRUE;
 		if (retries == MAX_RETRY)
@@ -566,13 +691,16 @@ doit(scgp)
 	params.askrange = TRUE;
 	params.name = "/dev/null";
 
-	for(;;) {
+	for (;;) {
 		if (!wait_unit_ready(scgp, 60))
 			comerrno(EX_BAD, "Device not ready.\n");
 
 		printf("0:read 1:veri   2:erase   3:read buffer 4:cache 5:ovtime 6:cap\n");
 		printf("7:wne  8:floppy 9:verify 10:checkcmds  11:read disk 12:write disk\n");
 		printf("13:scsireset 14:seektest 15: readda 16: reada 17: c2err\n");
+#ifdef	CLONE_WRITE
+		printf("18:readcd 19: lin 20: full toc\n");
+#endif
 
 		getint("Enter selection:", &i, 0, 20);
 		if (didintr)
@@ -586,6 +714,11 @@ doit(scgp)
 		case 15:	ra(scgp);		break;
 /*		case 16:	reada_disk(scgp, 0, 0);	break;*/
 		case 17:	readc2_disk(scgp, &params);	break;
+#ifdef	CLONE_WRITE
+		case 18:	readcd_disk(scgp, 0);	break;
+		case 19:	read_lin(scgp, 0);	break;
+		case 20:	read_ftoc(scgp, 0, FALSE);	break;
+#endif
 		}
 	}
 }
@@ -610,6 +743,314 @@ read_disk(scgp, parmp)
 	read_generic(scgp, parmp, fread_data, &rp, fdata_null);
 }
 
+#ifdef	CLONE_WRITE
+LOCAL void
+readcd_disk(scgp, parmp)
+	SCSI	*scgp;
+	parm_t	*parmp;
+{
+	rparm_t	rp;
+	int	osecsize = 2048;
+	int	oerr = 0;
+	int	oretr = 10;
+	int	(*funcp)__PR((SCSI *_scgp, rparm_t *_rp, caddr_t bp, long addr, int cnt));
+
+	scgp->silent++;
+	if (read_capacity(scgp) >= 0)
+		osecsize = scgp->cap->c_bsize;
+	scgp->silent--;
+	if (osecsize != 2048)
+		select_secsize(scgp, 2048);
+
+	read_capacity(scgp);
+	print_capacity(scgp, stderr);
+
+	rp.errors = 0;
+	rp.c2_errors = 0;
+	rp.c2_maxerrs = 0;
+	rp.c2_errsecs = 0;
+	rp.c2_badsecs = 0;
+	rp.secsize = 2448;
+	rp.ismmc = is_mmc(scgp, NULL, NULL);
+	funcp = fread_2448;
+
+	wait_unit_ready(scgp, 10);
+	if (fread_2448(scgp, &rp, Sbuf, 0, 0) < 0) {
+		errmsgno(EX_BAD, "read 2448 failed\n");
+		if (rp.ismmc &&
+		    fread_2448_16(scgp, &rp, Sbuf, 0, 0) >= 0) {
+			errmsgno(EX_BAD, "read 2448_16 : OK\n");
+
+			funcp = fread_2448_16;
+		}
+	}
+
+	oldmode(scgp, &oerr, &oretr);
+	exargs.oerr[0] = oerr;
+	exargs.oerr[1] = oretr;
+	exargs.oerr[2] = 0xFF;
+	if (parmp == NULL)		/* XXX Nur am Anfang!!! */
+		domode(scgp, -1, -1);
+	else
+		domode(scgp, nocorr?0x21:0x20, 10);
+
+	read_generic(scgp, parmp, funcp, &rp, fdata_null);
+	if (osecsize != 2048)
+		select_secsize(scgp, osecsize);
+	domode(scgp, oerr, oretr);
+}
+
+/* ARGSUSED */
+LOCAL void
+read_lin(scgp, parmp)
+	SCSI	*scgp;
+	parm_t	*parmp;
+{
+	parm_t	parm;
+	rparm_t	rp;
+
+	read_capacity(scgp);
+	print_capacity(scgp, stderr);
+
+	parm.start = ULONG_C(0xF0000000);
+	parm.end =   ULONG_C(0xFF000000);
+	parm.name = "DDD";
+
+	rp.errors = 0;
+	rp.c2_errors = 0;
+	rp.c2_maxerrs = 0;
+	rp.c2_errsecs = 0;
+	rp.c2_badsecs = 0;
+	rp.secsize = 2448;
+	rp.ismmc = is_mmc(scgp, NULL, NULL);
+	domode(scgp, -1, -1);
+	read_generic(scgp, &parm, fread_lin, &rp, fdata_null);
+}
+
+LOCAL int
+read_secheader(scgp, addr)
+	SCSI	*scgp;
+	long	addr;
+{
+	rparm_t	rp;
+	int	osecsize = 2048;
+	int	ret = 0;
+
+	scgp->silent++;
+	if (read_capacity(scgp) >= 0)
+		osecsize = scgp->cap->c_bsize;
+	scgp->silent--;
+	if (osecsize != 2048)
+		select_secsize(scgp, 2048);
+
+	read_capacity(scgp);
+
+	rp.errors = 0;
+	rp.c2_errors = 0;
+	rp.c2_maxerrs = 0;
+	rp.c2_errsecs = 0;
+	rp.c2_badsecs = 0;
+	rp.secsize = 2352;
+	rp.ismmc = is_mmc(scgp, NULL, NULL);
+
+	wait_unit_ready(scgp, 10);
+
+	fillbytes(Sbuf, 2352, '\0');
+	if (fread_2352(scgp, &rp, Sbuf, addr, 1) < 0) {
+		ret = -1;
+	}
+	if (osecsize != 2048)
+		select_secsize(scgp, osecsize);
+	return (ret);
+}
+
+/* ARGSUSED */
+LOCAL int
+read_ftoc(scgp, parmp, do_sectype)
+	SCSI	*scgp;
+	parm_t	*parmp;
+	BOOL	do_sectype;
+{
+	FILE	*f;
+	int	i;
+	char	filename[1024];
+	struct	tocheader *tp;
+	char	*p;
+	char	xb[256];
+	int	len;
+	char	xxb[10000];
+
+
+	strcpy(filename, "toc.dat");
+	if (strcmp(parmp->name, "/dev/null") != 0) {
+
+		len = strlen(parmp->name);
+		if (len > (sizeof (filename)-5)) {
+			len = sizeof (filename)-5;
+		}
+		js_snprintf(filename, sizeof (filename), "%.*s.toc", len, parmp->name);
+	}
+
+	tp = (struct tocheader *)xb;
+
+	fillbytes((caddr_t)xb, sizeof (xb), '\0');
+	if (read_toc(scgp, xb, 0, sizeof (struct tocheader), 0, FMT_FULLTOC) < 0) {
+		if (scgp->silent == 0 || scgp->verbose > 0)
+			errmsgno(EX_BAD, "Cannot read TOC header\n");
+		return (-1);
+	}
+	len = a_to_u_2_byte(tp->len) + sizeof (struct tocheader)-2;
+	error("TOC len: %d. First Session: %d Last Session: %d.\n", len, tp->first, tp->last);
+
+	if (read_toc(scgp, xxb, 0, len, 0, FMT_FULLTOC) < 0) {
+		if (len & 1) {
+			/*
+			 * Work around a bug in some operating systems that do not
+			 * handle odd byte DMA correctly for ATAPI drives.
+			 */
+			wait_unit_ready(scgp, 30);
+			read_toc(scgp, xb, 0, sizeof (struct tocheader), 0, FMT_FULLTOC);
+			wait_unit_ready(scgp, 30);
+			if (read_toc(scgp, xxb, 0, len+1, 0, FMT_FULLTOC) >= 0) {
+				goto itworked;
+			}
+		}
+		if (scgp->silent == 0)
+			errmsgno(EX_BAD, "Cannot read full TOC\n");
+		return (-1);
+	}
+
+itworked:
+	f = fileopen(filename, "wctb");
+
+	if (f == NULL)
+		comerr("Cannot open '%s'.\n", filename);
+	filewrite(f, xxb, len);
+	if (do_sectype)
+		read_sectypes(scgp, f);
+	fflush(f);
+	fclose(f);
+
+	p = &xxb[4];
+	for (; p < &xxb[len]; p += 11) {
+		for (i = 0; i < 11; i++)
+			error("%02X ", p[i] & 0xFF);
+		error("\n");
+	}
+	/*
+	 * List all lead out start times to give information about multi
+	 * session disks.
+	 */
+	p = &xxb[4];
+	for (; p < &xxb[len]; p += 11) {
+		if ((p[3] & 0xFF) == 0xA2) {
+			error("Lead out %d: %ld\n", p[0], msf_to_lba(p[8], p[9], p[10], TRUE));
+		}
+	}
+	return (0);
+}
+
+LOCAL void
+read_sectypes(scgp, f)
+	SCSI	*scgp;
+	FILE	*f;
+{
+	char	sect;
+
+	sect = SECT_AUDIO;
+	get_sectype(scgp, 4, &sect);
+	if (f != NULL)
+		filewrite(f, &sect, 1);
+	if (xdebug)
+		scg_prbytes("sec 0", (Uchar *)Sbuf, 16);
+
+	sect = SECT_AUDIO;
+	get_sectype(scgp, scgp->cap->c_baddr-4, &sect);
+	if (f != NULL)
+		filewrite(f, &sect, 1);
+	if (xdebug) {
+		scg_prbytes("sec E", (Uchar *)Sbuf, 16);
+		error("baddr: %ld\n", (long)scgp->cap->c_baddr);
+	}
+}
+
+LOCAL void
+get_sectype(scgp, addr, st)
+	SCSI	*scgp;
+	long	addr;
+	char	*st;
+{
+	char	*synchdr = "\0\377\377\377\377\377\377\377\377\377\377\0";
+	int	sectype = SECT_AUDIO;
+	int	i;
+	long	raddr = addr;
+#define	_MAX_TRY_	20
+
+	scgp->silent++;
+	for (i = 0; i < _MAX_TRY_ && read_secheader(scgp, raddr) < 0; i++) {
+		if (addr == 0)
+			raddr++;
+		else
+			raddr--;
+	}
+	scgp->silent--;
+	if (i >= _MAX_TRY_) {
+		error("Sectype (%ld) is CANNOT\n", addr);
+		return;
+	} else if (i > 0) {
+		error("Sectype (%ld) needed %d retries\n", addr, i);
+	}
+#undef	_MAX_TRY_
+
+	if (cmpbytes(Sbuf, synchdr, 12) < 12) {
+		if (xdebug)
+			error("Sectype (%ld) is AUDIO\n", addr);
+		if (st)
+			*st = SECT_AUDIO;
+		return;
+	}
+	if (xdebug)
+		error("Sectype (%ld) is DATA\n", addr);
+	if (Sbuf[15] == 0) {
+		if (xdebug)
+			error("Sectype (%ld) is MODE 0\n", addr);
+		sectype = SECT_MODE_0;
+
+	} else if (Sbuf[15] == 1) {
+		if (xdebug)
+			error("Sectype (%ld) is MODE 1\n", addr);
+		sectype = SECT_ROM;
+
+	} else if (Sbuf[15] == 2) {
+		if (xdebug)
+			error("Sectype (%ld) is MODE 2\n", addr);
+
+		if ((Sbuf[16+2]  & 0x20) == 0 &&
+		    (Sbuf[16+4+2]  & 0x20) == 0) {
+			if (xdebug)
+				error("Sectype (%ld) is MODE 2 form 1\n", addr);
+			sectype = SECT_MODE_2_F1;
+
+		} else if ((Sbuf[16+2]  & 0x20) != 0 &&
+		    (Sbuf[16+4+2]  & 0x20) != 0) {
+			if (xdebug)
+				error("Sectype (%ld) is MODE 2 form 2\n", addr);
+			sectype = SECT_MODE_2_F2;
+		} else {
+			if (xdebug)
+				error("Sectype (%ld) is MODE 2 formless\n", addr);
+			sectype = SECT_MODE_2;
+		}
+	} else {
+		error("Sectype (%ld) is UNKNOWN\n", addr);
+	}
+	if (st)
+		*st = sectype;
+	if (xdebug)
+		error("Sectype (%ld) is 0x%02X\n", addr, sectype);
+}
+
+#endif	/* CLONE_WRITE */
 
 char	zeroblk[512];
 
@@ -625,7 +1066,7 @@ readc2_disk(scgp, parmp)
 
 	scgp->silent++;
 	if (read_capacity(scgp) >= 0)
-		 osecsize = scgp->cap->c_bsize;
+		osecsize = scgp->cap->c_bsize;
 	scgp->silent--;
 	if (osecsize != 2048)
 		select_secsize(scgp, 2048);
@@ -671,6 +1112,117 @@ fread_data(scgp, rp, bp, addr, cnt)
 	return (read_g1(scgp, bp, addr, cnt));
 }
 
+#ifdef	CLONE_WRITE
+LOCAL int
+fread_2448(scgp, rp, bp, addr, cnt)
+	SCSI	*scgp;
+	rparm_t	*rp;
+	caddr_t	bp;
+	long	addr;
+	int	cnt;
+{
+	if (rp->ismmc) {
+		return (read_cd(scgp, bp, addr, cnt, rp->secsize,
+			/* Sync + all headers + user data + EDC/ECC */
+			(1 << 7 | 3 << 5 | 1 << 4 | 1 << 3),
+			/* plus all subchannels RAW */
+			1));
+	} else {
+		return (read_da(scgp, bp, addr, cnt, rp->secsize,
+			/* Sync + all headers + user data + EDC/ECC + all subch */
+			0x02));
+	}
+}
+
+LOCAL int
+fread_2448_16(scgp, rp, bp, addr, cnt)
+	SCSI	*scgp;
+	rparm_t	*rp;
+	caddr_t	bp;
+	long	addr;
+	int	cnt;
+{
+
+	if (rp->ismmc) {
+		track_t trackdesc;
+		int	ret;
+		int	i;
+		char	*p;
+
+		trackdesc.isecsize = 2368;
+		trackdesc.secsize = 2448;
+		ret = read_cd(scgp, bp, addr, cnt, 2368,
+			/* Sync + all headers + user data + EDC/ECC */
+			(1 << 7 | 3 << 5 | 1 << 4 | 1 << 3),
+			/* subchannels P/Q */
+			2);
+		if (ret < 0)
+			return (ret);
+
+		scatter_secs(&trackdesc, bp, cnt);
+		for (i = 0, p = bp+2352; i < cnt; i++) {
+#ifdef	more_than_q_sub
+			if ((p[15] & 0x80) != 0)
+				printf("P");
+#endif
+			/*
+			 * As the drives don't return P-sub, we check
+			 * whether the index equals 0.
+			 */
+			qpto96((Uchar *)p, (Uchar *)p, p[2] == 0);
+			p += 2448;
+		}
+		return (ret);
+	} else {
+		comerrno(EX_BAD, "Cannot fread_2448_16 on non MMC drives\n");
+
+		return (read_da(scgp, bp, addr, cnt, rp->secsize,
+			/* Sync + all headers + user data + EDC/ECC + all subch */
+			0x02));
+	}
+}
+
+LOCAL int
+fread_2352(scgp, rp, bp, addr, cnt)
+	SCSI	*scgp;
+	rparm_t	*rp;
+	caddr_t	bp;
+	long	addr;
+	int	cnt;
+{
+	if (rp->ismmc) {
+		return (read_cd(scgp, bp, addr, cnt, rp->secsize,
+			/* Sync + all headers + user data + EDC/ECC */
+			(1 << 7 | 3 << 5 | 1 << 4 | 1 << 3),
+			/* NO subchannels */
+			0));
+	} else {
+		comerrno(EX_BAD, "Cannot fread_2352 on non MMC drives\n");
+
+		return (read_da(scgp, bp, addr, cnt, rp->secsize,
+			/* Sync + all headers + user data + EDC/ECC + all subch */
+			0x02));
+	}
+}
+
+LOCAL int
+fread_lin(scgp, rp, bp, addr, cnt)
+	SCSI	*scgp;
+	rparm_t	*rp;
+	caddr_t	bp;
+	long	addr;
+	int	cnt;
+{
+	if (addr != ULONG_C(0xF0000000))
+		addr = ULONG_C(0xFFFFFFFF);
+
+	return (read_cd(scgp, bp, addr, cnt, rp->secsize,
+		/* Sync + all headers + user data + EDC/ECC */
+		(1 << 7 | 3 << 5 | 1 << 4 | 1 << 3),
+		/* plus all subchannels RAW */
+		1));
+}
+#endif	/* CLONE_WRITE */
 
 LOCAL int
 bits(c)
@@ -697,7 +1249,7 @@ bits(c)
 	return (n);
 }
 
-LOCAL int 
+LOCAL int
 bitidx(c)
 	int	c;
 {
@@ -733,7 +1285,7 @@ fread_c2(scgp, rp, bp, addr, cnt)
 			/* Sync + all headers + user data + EDC/ECC + C2 */
 /*			(1 << 7 | 3 << 5 | 1 << 4 | 1 << 3 | 2 << 1),*/
 			(1 << 7 | 3 << 5 | 1 << 4 | 1 << 3 | 1 << 1),
-        		/* without subchannels */
+			/* without subchannels */
 			0));
 	} else {
 		return (read_da(scgp, bp, addr, cnt, rp->secsize,
@@ -767,12 +1319,12 @@ fdata_c2(rp, bp, addr, cnt)
 
 	p = &bp[2352];
 
-	for (i=0; i < cnt; i++, p += (2352+294)) {
+	for (i = 0; i < cnt; i++, p += (2352+294)) {
 /*		scg_prbytes("XXX ", p, 294);*/
 		if ((j = cmpbytes(p, zeroblk, 294)) < 294) {
 			printf("C2 in sector: %3ld first at byte: %4d (0x%02X)", addr+i,
 				j*8 + bitidx(p[j]), p[j]&0xFF);
-			for (j=0,k=0; j < 294; j++)
+			for (j = 0, k = 0; j < 294; j++)
 				k += bits(p[j]);
 			printf(" total: %4d errors\n", k);
 /*			scg_prbytes("XXX ", p, 294);*/
@@ -783,7 +1335,7 @@ fdata_c2(rp, bp, addr, cnt)
 			if (k >= 100)
 				rp->c2_badsecs += 1;
 		}
-	}	
+	}
 	return (0);
 }
 
@@ -797,7 +1349,7 @@ read_scsi_g1(scgp, bp, addr, cnt)
 {
 	register struct	scg_cmd	*scmd = scgp->scmd;
 
-	fillbytes((caddr_t)scmd, sizeof(*scmd), '\0');
+	fillbytes((caddr_t)scmd, sizeof (*scmd), '\0');
 	scmd->addr = bp;
 /*	scmd->size = cnt*512;*/
 	scmd->size = cnt*scgp->cap->c_bsize;
@@ -808,7 +1360,7 @@ read_scsi_g1(scgp, bp, addr, cnt)
 	scmd->cdb.g1_cdb.lun = scg_lun(scgp);
 	g1_cdbaddr(&scmd->cdb.g1_cdb, addr);
 	g1_cdblen(&scmd->cdb.g1_cdb, cnt);
-	
+
 	scgp->cmdname = "read extended";
 
 	return (scg_cmd(scgp));
@@ -824,10 +1376,10 @@ write_scsi(scgp, bp, addr, cnt)
 	long	addr;
 	int	cnt;
 {
-	if(addr <= G0_MAXADDR)
-		return(write_g0(scgp, bp, addr, cnt));
+	if (addr <= G0_MAXADDR)
+		return (write_g0(scgp, bp, addr, cnt));
 	else
-		return(write_g1(scgp, bp, addr, cnt));
+		return (write_g1(scgp, bp, addr, cnt));
 }
 
 EXPORT int
@@ -842,7 +1394,7 @@ write_g0(scgp, bp, addr, cnt)
 	if (scgp->cap->c_bsize <= 0)
 		raisecond("capacity_not_set", 0L);
 
-	fillbytes((caddr_t)scmd, sizeof(*scmd), '\0');
+	fillbytes((caddr_t)scmd, sizeof (*scmd), '\0');
 	scmd->addr = bp;
 	scmd->size = cnt*scgp->cap->c_bsize;
 	scmd->flags = SCG_DISRE_ENA;
@@ -852,7 +1404,7 @@ write_g0(scgp, bp, addr, cnt)
 	scmd->cdb.g0_cdb.lun = scg_lun(scgp);
 	g0_cdbaddr(&scmd->cdb.g0_cdb, addr);
 	scmd->cdb.g0_cdb.count = (Uchar)cnt;
-	
+
 	scgp->cmdname = "write_g0";
 
 	return (scg_cmd(scgp));
@@ -870,7 +1422,7 @@ write_g1(scgp, bp, addr, cnt)
 	if (scgp->cap->c_bsize <= 0)
 		raisecond("capacity_not_set", 0L);
 
-	fillbytes((caddr_t)scmd, sizeof(*scmd), '\0');
+	fillbytes((caddr_t)scmd, sizeof (*scmd), '\0');
 	scmd->addr = bp;
 	scmd->size = cnt*scgp->cap->c_bsize;
 	scmd->flags = SCG_DISRE_ENA;
@@ -880,7 +1432,7 @@ write_g1(scgp, bp, addr, cnt)
 	scmd->cdb.g1_cdb.lun = scg_lun(scgp);
 	g1_cdbaddr(&scmd->cdb.g1_cdb, addr);
 	g1_cdblen(&scmd->cdb.g1_cdb, cnt);
-	
+
 	scgp->cmdname = "write_g1";
 
 	return (scg_cmd(scgp));
@@ -899,26 +1451,26 @@ Xrequest_sense(scgp)
 
 	cmdsave = scgp->cmdname;
 
-	movebytes(scmd, &ocmd, sizeof(*scmd));
+	movebytes(scmd, &ocmd, sizeof (*scmd));
 
-	fillbytes((caddr_t)sense_buf, sizeof(sense_buf), '\0');
+	fillbytes((caddr_t)sense_buf, sizeof (sense_buf), '\0');
 
-	fillbytes((caddr_t)scmd, sizeof(*scmd), '\0');
+	fillbytes((caddr_t)scmd, sizeof (*scmd), '\0');
 	scmd->addr = (caddr_t)sense_buf;
-	scmd->size = sizeof(sense_buf);
+	scmd->size = sizeof (sense_buf);
 	scmd->flags = SCG_RECV_DATA|SCG_DISRE_ENA;
 	scmd->cdb_len = SC_G0_CDBLEN;
 	scmd->sense_len = CCS_SENSE_LEN;
 	scmd->cdb.g1_cdb.cmd = 0x3;
 	scmd->cdb.g1_cdb.lun = scg_lun(scgp);
-	scmd->cdb.g0_cdb.count = sizeof(sense_buf);
-	
+	scmd->cdb.g0_cdb.count = sizeof (sense_buf);
+
 	scgp->cmdname = "request sense";
 
 	scg_cmd(scgp);
 
-	sense_count = sizeof(sense_buf) - scg_getresid(scgp);
-	movebytes(&ocmd, scmd, sizeof(*scmd));
+	sense_count = sizeof (sense_buf) - scg_getresid(scgp);
+	movebytes(&ocmd, scmd, sizeof (*scmd));
 	scmd->sense_count = sense_count;
 	movebytes(sense_buf, (Uchar *)&scmd->sense, scmd->sense_count);
 
@@ -943,7 +1495,7 @@ read_retry(scgp, bp, addr, cnt, rfunc, rp)
 	int	err;
 	char	dummybuf[8192];
 
-	if (secsize > sizeof(dummybuf)) {
+	if (secsize > sizeof (dummybuf)) {
 		errmsgno(EX_BAD, "Cannot retry, sector size %d too big.\n", secsize);
 		return (-1);
 	}
@@ -991,6 +1543,10 @@ read_retry(scgp, bp, addr, cnt, rfunc, rp)
 /*				error("\n");*/
 /*				errmsgno(err, "Cannot read source disk\n");*/
 			} else {
+				if (scg_getresid(scgp)) {
+					error("\nresid: %d\n", scg_getresid(scgp));
+					return (-1);
+				}
 				break;
 			}
 		} while (++try < retries);
@@ -1001,12 +1557,17 @@ read_retry(scgp, bp, addr, cnt, rfunc, rp)
 					addr, ++rp->errors);
 
 			if (scgp->silent <= 1 && lverbose > 0)
-				scg_printerr(scgp); 
+				scg_printerr(scgp);
+
+			add_bad(addr);
 
 			if (!noerror)
 				return (-1);
 			errmsgno(EX_BAD, "-noerror set, continuing ...\n");
 		} else {
+			if (try >= maxtry)
+				maxtry = try;
+
 			if (try > 1) {
 				error("\n");
 				errmsgno(EX_BAD,
@@ -1034,15 +1595,21 @@ read_generic(scgp, parmp, rfunc, rp, dfunc)
 	char	*defname = NULL;
 	FILE	*f;
 	long	addr = 0L;
+	long	old_addr = 0L;
 	long	num;
 	long	end = 0L;
 	long	start = 0L;
 	long	cnt = 0L;
+	long	next_point = 0L;
+	long	secs_per_point = 0L;
+	double  speed;
 	int	msec;
+	int	old_msec = 0;
 	int	err = 0;
 	BOOL	askrange = FALSE;
 	BOOL	isrange = FALSE;
 	int	secsize = rp->secsize;
+	int	i = 0;
 
 	if (is_suid) {
 		if (scgp->inq->type != INQ_ROMD)
@@ -1054,8 +1621,8 @@ read_generic(scgp, parmp, rfunc, rp, dfunc)
 	if (parmp != NULL && !askrange && (parmp->start <= parmp->end))
 		isrange = TRUE;
 
-	filename[0] ='\0';
-	
+	filename[0] = '\0';
+
 	scgp->silent++;
 	if (read_capacity(scgp) >= 0)
 		end = scgp->cap->c_baddr + 1;
@@ -1083,8 +1650,8 @@ read_generic(scgp, parmp, rfunc, rp, dfunc)
 		defname = "disk.out";
 		error("Copy from SCSI (%d,%d,%d) disk to file\n",
 					scg_scsibus(scgp), scg_target(scgp), scg_lun(scgp));
-		error("Enter filename [%s]: ", defname);flush();
-		(void)getline(filename, sizeof(filename));
+		error("Enter filename [%s]: ", defname); flush();
+		(void) getline(filename, sizeof (filename));
 	}
 
 	if (askrange) {
@@ -1107,11 +1674,11 @@ read_generic(scgp, parmp, rfunc, rp, dfunc)
 	}
 
 	if (filename[0] == '\0')
-		strncpy(filename, defname, sizeof(filename));
-	filename[sizeof(filename)-1] = '\0';
+		strncpy(filename, defname, sizeof (filename));
+	filename[sizeof (filename)-1] = '\0';
 	if (streql(filename, "-")) {
 		f = stdout;
-#if	defined(__CYGWIN32__) || defined(__EMX__)
+#ifdef	NEED_O_BINARY
 		setmode(STDOUT_FILENO, O_BINARY);
 #endif
 	} else if ((f = fileopen(filename, notrunc?"wcub":"wctub")) == NULL)
@@ -1121,13 +1688,48 @@ read_generic(scgp, parmp, rfunc, rp, dfunc)
 	if (gettimeofday(&starttime, (struct timezone *)0) < 0)
 		comerr("Cannot get start time\n");
 
-	for(;addr < end; addr += cnt) {
+	if (meshpoints > 0) {
+		if ((end-start) < meshpoints)
+			secs_per_point = 1;
+		else
+			secs_per_point = (end-start) / meshpoints;
+		next_point = start + secs_per_point;
+		old_addr = start;
+	}
+
+	for (; addr < end; addr += cnt) {
 		if (didintr)
 			comexit(exsig);		/* XXX besseres Konzept?!*/
 
 		if ((addr + cnt) > end)
 			cnt = end - addr;
 
+		if (meshpoints > 0) {
+			if (addr > next_point) {
+
+				msec = prstats_silent();
+				if ((msec - old_msec) == 0)		/* Avoid division by zero */
+					msec = old_msec + 1;
+				speed = ((addr - old_addr)/(1000.0/secsize)) / (0.001*(msec - old_msec));
+				if (do_factor) {
+					if (is_cdrom)
+						speed /= 176.400;
+					else if (is_dvd)
+						speed /= 1385.0;
+				}
+				error("addr: %8ld cnt: %ld", addr, cnt);
+				printf("%8ld %8.2f\n", addr, speed);
+				error("\r");
+				next_point += secs_per_point;
+				old_addr = addr;
+				old_msec = msec;
+				i++;
+				if (meshpoints < 100)
+					flush();
+				else if (i % (meshpoints/100) == 0)
+					flush();
+			}
+		}
 		error("addr: %8ld cnt: %ld\r", addr, cnt);
 
 		scgp->silent++;
@@ -1145,6 +1747,10 @@ read_generic(scgp, parmp, rfunc, rp, dfunc)
 				goto out;
 		} else {
 			scgp->silent--;
+			if (scg_getresid(scgp)) {
+				error("\nresid: %d\n", scg_getresid(scgp));
+				goto out;
+			}
 		}
 		(*dfunc)(rp, Sbuf, addr, cnt);
 		if (filewrite(f, Sbuf, cnt * secsize) < 0) {
@@ -1169,6 +1775,7 @@ out:
 		(double)(addr - start)/(1024.0/secsize),
 		(double)((addr - start)/(1024.0/secsize)) / (0.001*msec));
 #endif
+	print_bad();
 }
 
 LOCAL void
@@ -1189,13 +1796,13 @@ write_disk(scgp, parmp)
 	if (is_suid)
 		comerrno(EX_BAD, "Not root. Will not write in suid mode\n");
 
-	filename[0] ='\0';
+	filename[0] = '\0';
 	if (read_capacity(scgp) >= 0) {
 		end = scgp->cap->c_baddr + 1;
 		print_capacity(scgp, stderr);
 	}
 
-	if (end <= 0)
+	if (end <= 1)
 		end = 10000000;	/* Hack to write empty disks */
 
 	if (parmp) {
@@ -1209,32 +1816,30 @@ write_disk(scgp, parmp)
 		if (parmp->end != -1 && parmp->end < end)
 			end = parmp->end;
 		cnt = Sbufsize / scgp->cap->c_bsize;
-	}
-else {
-
-	error("Copy from file to SCSI (%d,%d,%d) disk\n",
+	} else {
+		error("Copy from file to SCSI (%d,%d,%d) disk\n",
 					scg_scsibus(scgp), scg_target(scgp), scg_lun(scgp));
-	error("Enter filename [%s]: ", defname);flush();
-	(void)getline(filename, sizeof(filename));
-	error("Notice: reading from file always starts at file offset 0.\n");
+		error("Enter filename [%s]: ", defname); flush();
+		(void) getline(filename, sizeof (filename));
+		error("Notice: reading from file always starts at file offset 0.\n");
 
-	getlong("Enter starting sector for copy:", &addr, 0L, end-1);
-	start = addr;
-	cnt = end - addr;
-	getlong("Enter number of sectors to copy:", &end, 1L, end);
-	end = addr + cnt;
+		getlong("Enter starting sector for copy:", &addr, 0L, end-1);
+		start = addr;
+		cnt = end - addr;
+		getlong("Enter number of sectors to copy:", &end, 1L, end);
+		end = addr + cnt;
 
-	cnt = Sbufsize / scgp->cap->c_bsize;
-	getlong("Enter number of sectors per copy:", &cnt, 1L, cnt);
-/*	error("end:  %8ld\n", end);*/
-}
+		cnt = Sbufsize / scgp->cap->c_bsize;
+		getlong("Enter number of sectors per copy:", &cnt, 1L, cnt);
+/*		error("end:  %8ld\n", end);*/
+	}
 
 	if (filename[0] == '\0')
-		strncpy(filename, defname, sizeof(filename));
-	filename[sizeof(filename)-1] = '\0';
+		strncpy(filename, defname, sizeof (filename));
+	filename[sizeof (filename)-1] = '\0';
 	if (streql(filename, "-")) {
 		f = stdin;
-#if	defined(__CYGWIN32__) || defined(__EMX__)
+#ifdef	NEED_O_BINARY
 		setmode(STDIN_FILENO, O_BINARY);
 #endif
 	} else if ((f = fileopen(filename, "rub")) == NULL)
@@ -1244,7 +1849,7 @@ else {
 	if (gettimeofday(&starttime, (struct timezone *)0) < 0)
 		comerr("Cannot get start time\n");
 
-	for(;addr < end; addr += cnt) {
+	for (; addr < end; addr += cnt) {
 		if (didintr)
 			comexit(exsig);		/* XXX besseres Konzept?!*/
 
@@ -1279,12 +1884,12 @@ choice(n)
 #if	defined(HAVE_DRAND48)
 	extern	double	drand48 __PR((void));
 
-	return drand48() * n;
+	return (drand48() * n);
 #else
 #	if	defined(HAVE_RAND)
 	extern	int	rand __PR((void));
 
-	return rand() % n;
+	return (rand() % n);
 #	else
 	return (0);
 #	endif
@@ -1312,13 +1917,13 @@ ra(scgp)
 		errmsg("read CD\n");
 	f = fileopen("DDA", "wctb");
 /*	filewrite(f, Sbuf, 50 * 2352 - scg_getresid(scgp));*/
-	filewrite(f, Sbuf, 50 * 2352 );
+	filewrite(f, Sbuf, 50 * 2352);
 	fclose(f);
 }
 
 #define	g5x_cdblen(cdb, len)	((cdb)->count[0] = ((len) >> 16L)& 0xFF,\
-				 (cdb)->count[1] = ((len) >> 8L) & 0xFF,\
-				 (cdb)->count[2] = (len) & 0xFF)
+				(cdb)->count[1] = ((len) >> 8L) & 0xFF,\
+				(cdb)->count[2] = (len) & 0xFF)
 
 EXPORT int
 read_da(scgp, bp, addr, cnt, framesize, subcode)
@@ -1334,7 +1939,7 @@ read_da(scgp, bp, addr, cnt, framesize, subcode)
 	if (scgp->cap->c_bsize <= 0)
 		raisecond("capacity_not_set", 0L);
 
-	fillbytes((caddr_t)scmd, sizeof(*scmd), '\0');
+	fillbytes((caddr_t)scmd, sizeof (*scmd), '\0');
 	scmd->addr = bp;
 	scmd->size = cnt*framesize;
 	scmd->flags = SCG_RECV_DATA|SCG_DISRE_ENA;
@@ -1345,7 +1950,7 @@ read_da(scgp, bp, addr, cnt, framesize, subcode)
 	g5_cdbaddr(&scmd->cdb.g5_cdb, addr);
 	g5_cdblen(&scmd->cdb.g5_cdb, cnt);
 	scmd->cdb.g5_cdb.res10 = subcode;
-	
+
 	scgp->cmdname = "read_da";
 
 	return (scg_cmd(scgp));
@@ -1363,7 +1968,7 @@ read_cd(scgp, bp, addr, cnt, framesize, data, subch)
 {
 	register struct	scg_cmd	*scmd = scgp->scmd;
 
-	fillbytes((caddr_t)scmd, sizeof(*scmd), '\0');
+	fillbytes((caddr_t)scmd, sizeof (*scmd), '\0');
 	scmd->addr = bp;
 	scmd->size = cnt*framesize;
 	scmd->flags = SCG_RECV_DATA|SCG_DISRE_ENA;
@@ -1371,12 +1976,12 @@ read_cd(scgp, bp, addr, cnt, framesize, data, subch)
 	scmd->sense_len = CCS_SENSE_LEN;
 	scmd->cdb.g5_cdb.cmd = 0xBE;
 	scmd->cdb.g5_cdb.lun = scg_lun(scgp);
-        scmd->cdb.g5_cdb.res = 0;	/* expected sector type field ALL */
+	scmd->cdb.g5_cdb.res = 0;	/* expected sector type field ALL */
 	g5_cdbaddr(&scmd->cdb.g5_cdb, addr);
 	g5x_cdblen(&scmd->cdb.g5_cdb, cnt);
 
 	scmd->cdb.g5_cdb.count[3] = data & 0xFF;
-        scmd->cdb.g5_cdb.res10 = subch & 0x07;
+	scmd->cdb.g5_cdb.res10 = subch & 0x07;
 
 	scgp->cmdname = "read_cd";
 
@@ -1395,8 +2000,8 @@ oldmode(scgp, errp, retrp)
 	int	i;
 	int	len;
 
-	fillbytes(mode, sizeof(mode), '\0');
-	fillbytes(cmode, sizeof(cmode), '\0');
+	fillbytes(mode, sizeof (mode), '\0');
+	fillbytes(cmode, sizeof (cmode), '\0');
 
 	if (!get_mode_params(scgp, 0x01, "CD error recovery parameter",
 			mode, (Uchar *)0, (Uchar *)cmode, (Uchar *)0, &len)) {
@@ -1435,8 +2040,8 @@ domode(scgp, err, retr)
 	int	i;
 	int	len;
 
-	fillbytes(mode, sizeof(mode), '\0');
-	fillbytes(cmode, sizeof(cmode), '\0');
+	fillbytes(mode, sizeof (mode), '\0');
+	fillbytes(cmode, sizeof (cmode), '\0');
 
 	if (!get_mode_params(scgp, 0x01, "CD error recovery parameter",
 			mode, (Uchar *)0, (Uchar *)cmode, (Uchar *)0, &len)) {
@@ -1489,7 +2094,7 @@ LOCAL	void	qpto96		__PR((Uchar *sub, Uchar *subq, int dop));
 /*
  * Q-Sub auf 96 Bytes blhen und P-Sub addieren
  *
- * OUT: sub, IN: subqptr 
+ * OUT: sub, IN: subqptr
  */
 LOCAL void
 /*EXPORT void*/
@@ -1509,10 +2114,10 @@ qpto96(sub, subqptr, dop)
 	}
 	fillbytes(sub, 96, '\0');
 
-	if (dop) for (i=0, p = sub; i < 96; i++) {
+	if (dop) for (i = 0, p = sub; i < 96; i++) {
 		*p++ |= 0x80;
 	}
-	for (i=0, p = sub; i < 12; i++) {
+	for (i = 0, p = sub; i < 12; i++) {
 		c = subqptr[i] & 0xFF;
 /*printf("%02X\n", c);*/
 		if (c & 0x80)
@@ -1559,7 +2164,7 @@ ovtime(scgp)
 	register int	i;
 
 	scgp->silent++;
-	(void)test_unit_ready(scgp);
+	(void) test_unit_ready(scgp);
 	scgp->silent--;
 	if (test_unit_ready(scgp) < 0)
 		return;
@@ -1569,8 +2174,8 @@ ovtime(scgp)
 	if (gettimeofday(&starttime, (struct timezone *)0) < 0)
 		comerr("Cannot get start time\n");
 
-	for (i = 1000; --i >= 0;) {
-		(void)test_unit_ready(scgp);
+	for (i = 1000; --i >= 0; ) {
+		(void) test_unit_ready(scgp);
 
 		if (didintr)
 			return;
@@ -1591,8 +2196,8 @@ ovtime(scgp)
 		if (gettimeofday(&starttime, (struct timezone *)0) < 0)
 			comerr("Cannot get start time\n");
 
-		for (i = 1000; --i >= 0;) {
-			(void)seek_g0(scgp, 0L);
+		for (i = 1000; --i >= 0; ) {
+			(void) seek_g0(scgp, 0L);
 
 			if (didintr)
 				return;
@@ -1612,8 +2217,8 @@ ovtime(scgp)
 	if (gettimeofday(&starttime, (struct timezone *)0) < 0)
 		comerr("Cannot get start time\n");
 
-	for (i = 1000; --i >= 0;) {
-		(void)seek_g1(scgp, 0L);
+	for (i = 1000; --i >= 0; ) {
+		(void) seek_g1(scgp, 0L);
 
 		if (didintr)
 			return;
@@ -1621,3 +2226,41 @@ ovtime(scgp)
 
 	prstats();
 }
+
+#define	BAD_INC		16
+long	*badsecs;
+int	nbad;
+int	maxbad;
+
+LOCAL void
+add_bad(addr)
+	long	addr;
+{
+	if (maxbad == 0) {
+		maxbad = BAD_INC;
+		badsecs = malloc(maxbad * sizeof (long));
+		if (badsecs == NULL)
+			comerr("No memory for bad sector list\n.");
+	}
+	if (nbad >= maxbad) {
+		maxbad += BAD_INC;
+		badsecs = realloc(badsecs, maxbad * sizeof (long));
+		if (badsecs == NULL)
+			comerr("No memory to grow bad sector list\n.");
+	}
+	badsecs[nbad++] = addr;
+}
+
+LOCAL void
+print_bad()
+{
+	int	i;
+
+	if (nbad == 0)
+		return;
+
+	error("Max corected retry count was %d (limited to %d).\n", maxtry, retries);
+	error("The following %d sector(s) could not be read correctly:\n", nbad);
+	for (i = 0; i < nbad; i++)
+		error("%ld\n", badsecs[i]);
+}
