@@ -1,7 +1,7 @@
-/* @(#)scgcheck.c	1.5 02/11/30 Copyright 1998-2002 J. Schilling */
+/* @(#)scgcheck.c	1.6 04/09/08 Copyright 1998-2002 J. Schilling */
 #ifndef lint
 static	char sccsid[] =
-	"@(#)scgcheck.c	1.5 02/11/30 Copyright 1998-2002 J. Schilling";
+	"@(#)scgcheck.c	1.6 04/09/08 Copyright 1998-2002 J. Schilling";
 #endif
 /*
  *	Copyright (c) 1998-2002 J. Schilling
@@ -47,6 +47,7 @@ LOCAL	SCSI	*doopen		__PR((char *dev));
 LOCAL	void	checkversion	__PR((SCSI *scgp));
 LOCAL	void	getbuf		__PR((SCSI *scgp));
 EXPORT	void	flushit		__PR((void));
+EXPORT	int	countopen	__PR((void));
 
 
 	char	*dev;
@@ -55,14 +56,16 @@ int		kdebug;		/* kernel debug messages */
 int		scsi_verbose;	/* SCSI verbose flag */
 int		lverbose;	/* local verbose flag */
 int		silent;		/* SCSI silent flag */
-int		deftimeout = 40;/* default SCSI timeout */
+int		deftimeout = 40; /* default SCSI timeout */
+int		xdebug;		/* extended debug flag */
 
 char	*buf;			/* The transfer buffer */
 long	bufsize;		/* The size of the transfer buffer */
 
 FILE	*logfile;
 char	unavail[] = "<data unavaiable>";
-char	scgc_version[] = "2.00.3";
+char	scgc_version[] = "2.01";
+int	basefds;
 
 #define	BUF_SIZE	(126*1024)
 #define	MAX_BUF_SIZE	(16*1024*1024)
@@ -86,7 +89,7 @@ usage(ret)
 	exit(ret);
 }	
 
-char	opts[]   = "debug#,d+,kdebug#,kd#,timeout#,verbose+,v+,Verbose+,V+,silent,s,help,h,version,dev*,f*";
+char	opts[]   = "debug#,d+,kdebug#,kd#,timeout#,verbose+,v+,Verbose+,V+,silent,s,x+,xd#,help,h,version,dev*,f*";
 
 EXPORT int
 main(ac, av)
@@ -95,13 +98,10 @@ main(ac, av)
 {
 	int	cac;
 	char	* const *cav;
-	SCSI	*scgp;
+	SCSI	*scgp = NULL;
 	char	device[128];
 	char	abuf[2];
 	int	ret;
-	int	scsibus;
-	int	target;
-	int	lun;
 	int	fcount;
 	BOOL	help = FALSE;
 	BOOL	pversion = FALSE;
@@ -112,13 +112,14 @@ main(ac, av)
 	cac = --ac;
 	cav = ++av;
 
-	if(getallargs(&cac, &cav, opts,
+	if (getallargs(&cac, &cav, opts,
 			&debug, &debug,
 			&kdebug, &kdebug,
 			&deftimeout,
 			&lverbose, &lverbose,
 			&scsi_verbose, &scsi_verbose,
 			&silent, &silent,
+			&xdebug, &xdebug,
 			&help, &help, &pversion,
 			&dev,
 			&filename) < 0) {
@@ -138,7 +139,7 @@ main(ac, av)
 	cac = ac;
 	cav = av;
 
-	while(getfiles(&cac, &cav, opts) > 0) {
+	while (getfiles(&cac, &cav, opts) > 0) {
 		fcount++;
 		cac--;
 		cav++;
@@ -166,18 +167,29 @@ Copyright (C) 1998,2001 Jrg Schilling\n",
 	 */
 	scg_remote();
 
+	basefds = countopen();
+	if (xdebug)
+		error("nopen: %d\n", basefds);
+
 	printf("Checking if your implementation supports to scan the SCSI bus.\n");
 	fprintf(logfile, "Checking if your implementation supports to scan the SCSI bus.\n");
 	printf("Trying to open device: '%s'.\n", dev);
 	fprintf(logfile, "Trying to open device: '%s'.\n", dev);
 	scgp = doopen(dev);
+
+	if (xdebug) {
+		error("nopen: %d\n", countopen());
+		error("Scanopen opened %d new files.\n", countopen() - basefds);
+	}
+
+	device[0] = '\0';
 	if (scgp == NULL) do {
 		error("SCSI open failed...\n");
 		if (!scg_yes("Retry with different device name? "))
 			break;
 		error("Enter SCSI device name for bus scanning [%s]: ", device);
 		flushit();
-		(void)getline(device, sizeof(device));
+		(void) getline(device, sizeof (device));
 		if (device[0] == '\0')
 			strcpy(device, "0,6,0");
 
@@ -192,6 +204,7 @@ Copyright (C) 1998,2001 Jrg Schilling\n",
 		ret = select_target(scgp, stdout);
 		select_target(scgp, logfile);
 		scg_close(scgp);
+		scgp = NULL;
 		if (ret < 1) {
 			printf("----------> SCSI scan bus test: found NO TARGETS\n");
 			fprintf(logfile, "----------> SCSI scan bus test: found NO TARGETS\n");
@@ -204,6 +217,8 @@ Copyright (C) 1998,2001 Jrg Schilling\n",
 		fprintf(logfile, "----------> SCSI scan bus test FAILED\n");
 	}
 
+	if (xdebug)
+		error("nopen: %d\n", countopen());
 	printf("For the next test we need to open a single SCSI device.\n");
 	fprintf(logfile, "For the next test we need to open a single SCSI device.\n");
 	printf("Best results will be obtained if you specify a modern CD-ROM drive.\n");
@@ -212,7 +227,7 @@ Copyright (C) 1998,2001 Jrg Schilling\n",
 	do {
 		error("Enter SCSI device name [%s]: ", device);
 		flushit();
-		(void)getline(device, sizeof(device));
+		(void) getline(device, sizeof (device));
 		if (device[0] == '\0')
 			strcpy(device, "0,6,0");
 
@@ -223,22 +238,115 @@ Copyright (C) 1998,2001 Jrg Schilling\n",
 			checkversion(scgp);
 			getbuf(scgp);
 		}
+		/*
+		 * XXX hier mu getestet werden ob das Gert brauchbar fr die folgenden Tests ist.
+		 */
 	} while (scgp == NULL);
+	if (xdebug)
+		error("nopen: %d\n", countopen());
 	/*
 	 * First try to check which type of SCSI device we
 	 * have.
 	 */
 	scgp->silent++;
-	(void)unit_ready(scgp);	/* eat up unit attention */
+	(void) unit_ready(scgp);	/* eat up unit attention */
 	scgp->silent--;
 	getdev(scgp, TRUE);
 	printinq(scgp, logfile);
 
+	printf("Ready to start test for second SCSI open? Enter <CR> to continue: ");
+	flushit();
+	(void) getline(abuf, sizeof (abuf));
+#define	CHECK_SECOND_OPEN
+#ifdef	CHECK_SECOND_OPEN
+	if (!streql(abuf, "n")) {
+		SCSI	*scgp2 = NULL;
+		int	oldopen = countopen();
+		BOOL	second_ok = TRUE;
+
+		scgp->silent++;
+		ret = inquiry(scgp, buf, sizeof (struct scsi_inquiry));
+		scgp->silent--;
+		if (xdebug)
+			error("ret: %d key: %d\n", ret, scg_sense_key(scgp));
+		if (ret >= 0 || scgp->scmd->error == SCG_RETRYABLE) {
+			printf("First SCSI open OK - device usable\n");
+			printf("Checking for second SCSI open.\n");
+			fprintf(logfile, "First SCSI open OK - device usable\n");
+			fprintf(logfile, "Checking for second SCSI open.\n");
+			if ((scgp2 = doopen(device)) != NULL) {
+				printf("Second SCSI open for same device succeeded, %d file descriptor(s) used.\n",
+					countopen() - oldopen);
+				fprintf(logfile,
+					"Second SCSI open for same device succeeded, %d file descriptor(s) used.\n",
+					countopen() - oldopen);
+				scgp->silent++;
+				ret = inquiry(scgp, buf, sizeof (struct scsi_inquiry));
+				scgp->silent--;
+				if (ret >= 0 || scgp->scmd->error == SCG_RETRYABLE) {
+					printf("Second SCSI open is usable\n");
+					fprintf(logfile, "Second SCSI open is usable\n");
+				}
+				printf("Closing second SCSI.\n");
+				fprintf(logfile, "Closing second SCSI.\n");
+				scg_close(scgp2);
+				scgp2 = NULL;
+				printf("Checking first SCSI.\n");
+				fprintf(logfile, "Checking first SCSI.\n");
+				scgp->silent++;
+				ret = inquiry(scgp, buf, sizeof (struct scsi_inquiry));
+				scgp->silent--;
+				if (ret >= 0 || scgp->scmd->error == SCG_RETRYABLE) {
+					printf("First SCSI open is still usable\n");
+					printf("Second SCSI open test passed.\n");
+					fprintf(logfile, "First SCSI open is still usable\n");
+					fprintf(logfile, "Second SCSI open test passed.\n");
+				} else if (ret < 0 && scgp->scmd->error == SCG_FATAL) {
+					second_ok = FALSE;
+					printf("First SCSI open does not work anymore.\n");
+					printf("Second SCSI open test FAILED.\n");
+					fprintf(logfile, "First SCSI open does not work anymore.\n");
+					fprintf(logfile, "Second SCSI open test FAILED.\n");
+				} else {
+					second_ok = FALSE;
+					printf("First SCSI open has strange problems.\n");
+					printf("Second SCSI open test FAILED.\n");
+					fprintf(logfile, "First SCSI open has strange problems.\n");
+					fprintf(logfile, "Second SCSI open test FAILED.\n");
+				}
+			} else {
+				second_ok = FALSE;
+				printf("Cannot open same SCSI device a second time.\n");
+				printf("Second SCSI open test FAILED.\n");
+				fprintf(logfile, "Cannot open same SCSI device a second time.\n");
+				fprintf(logfile, "Second SCSI open test FAILED.\n");
+			}
+		} else {
+			second_ok = FALSE;
+			printf("First SCSI open is not usable\n");
+			printf("Second SCSI open test FAILED.\n");
+			fprintf(logfile, "First SCSI open is not usable\n");
+			fprintf(logfile, "Second SCSI open test FAILED.\n");
+		}
+		if (!second_ok && scgp2) {
+			if (xdebug > 1)
+				error("scgp %p scgp2 %p\n", scgp, scgp2);
+			if (scgp)
+				scg_close(scgp);
+			if (scgp2)
+				scg_close(scgp2);
+			scgp = doopen(device);
+			if (xdebug > 1)
+				error("scgp %p\n", scgp);
+		}
+	}
+#endif	/* CHECK_SECOND_OPEN */
+
 	printf("Ready to start test for succeeded command? Enter <CR> to continue: ");
 	flushit();
-	(void)getline(abuf, sizeof(abuf));
+	(void) getline(abuf, sizeof (abuf));
 	scgp->verbose++;
-	ret = inquiry(scgp, buf, sizeof(struct scsi_inquiry));
+	ret = inquiry(scgp, buf, sizeof (struct scsi_inquiry));
 	scg_vsetup(scgp);
 	scg_errfflush(scgp, logfile);
 	scgp->verbose--;
@@ -279,7 +387,7 @@ doopen(devname)
 	SCSI	*scgp;
 	char	errstr[128];
 
-	if ((scgp = scg_open(devname, errstr, sizeof(errstr), debug, lverbose)) == (SCSI *)0) {
+	if ((scgp = scg_open(devname, errstr, sizeof (errstr), debug, lverbose)) == (SCSI *)0) {
 		errmsg("%s%sCannot open SCSI driver.\n", errstr, errstr[0]?". ":"");
 		fprintf(logfile, "%s. %s%sCannot open SCSI driver.\n",
 			errmsgstr(geterrno()), errstr, errstr[0]?". ":"");
@@ -304,7 +412,7 @@ checkversion(scgp)
 	char	*auth;
 
 	/*
-	 * Warning: you are not allowed to modify or to remove this 
+	 * Warning: you are not allowed to modify or to remove this
 	 * version checking code!
 	 */
 	vers = scg_version(0, SCG_VERSION);
@@ -378,3 +486,19 @@ flushit()
 	flush();
 	fflush(logfile);
 }
+
+/*--------------------------------------------------------------------------*/
+#include <fctldefs.h>
+
+int
+countopen()
+{
+	int	nopen = 0;
+	int	i;
+
+	for (i = 0; i < 1000; i++) {
+		if (fcntl(i, F_GETFD, 0) >= 0)
+			nopen++;
+	}
+	return (nopen);
+}
